/* This file is automatically generated by i386-gen.  Do not edit!  */
/* Copyright (C) 2007-2026 Free Software Foundation, Inc.

   This file is part of the GNU opcodes library.

   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   It is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
   MA 02110-1301, USA.  */

static const char i386_mnemonics[];

#define MN_ud0 0x1
#define MN_prefetcht0 0x5
#define MN_prefetchit0 0x10
#define MN_t2rpntlvwz0 0x1c
#define MN_xsha1 0x28
#define MN_fld1 0x2e
#define MN_ud1 0x33
#define MN_sha1msg1 0x37
#define MN_vsha512msg1 0x40
#define MN_vsm3msg1 0x4c
#define MN_sha256msg1 0x55
#define MN_fprem1 0x60
#define MN_f2xm1 0x67
#define MN_fyl2xp1 0x6d
#define MN_t2rpntlvwz0t1 0x75
#define MN_t2rpntlvwz1t1 0x83
#define MN_tileloaddt1 0x91
#define MN_prefetcht1 0x9d
#define MN_prefetchit1 0xa8
#define MN_pfrcpit1 0xb4
#define MN_pfrsqit1 0xbd
#define MN_int1 0xc6
#define MN_t2rpntlvwz0rst1 0xcb
#define MN_t2rpntlvwz1rst1 0xdb
#define MN_tileloaddrst1 0xeb
#define MN_prefetchwt1 0xf9
#define MN_t2rpntlvwz1 0x105
#define MN_xsha512 0x111
#define MN_vmovdqa32 0x119
#define MN_data32 0x123
#define MN_crc32 0x12a
#define MN_endbr32 0x130
#define MN_addr32 0x138
#define MN_vmovdqu32 0x13f
#define MN_ud2 0x149
#define MN_fldlg2 0x14d
#define MN_xrng2 0x154
#define MN_sha1msg2 0x15a
#define MN_vsha512msg2 0x163
#define MN_vsm3msg2 0x16f
#define MN_sha256msg2 0x178
#define MN_push2 0x183
#define MN_montmul2 0x189
#define MN_sm2 0x192
#define MN_fldln2 0x196
#define MN_pop2 0x19d
#define MN_vsha512rnds2 0x1a2
#define MN_vsm3rnds2 0x1af
#define MN_sha256rnds2 0x1b9
#define MN_prefetcht2 0x1c5
#define MN_pfrcpit2 0x1d0
#define MN_prefetchrst2 0x1d9
#define MN_vbroadcastf32x2 0x1e6
#define MN_vbroadcasti32x2 0x1f6
#define MN_vshuff64x2 0x206
#define MN_vextractf64x2 0x211
#define MN_vinsertf64x2 0x21f
#define MN_vbroadcastf64x2 0x22c
#define MN_vshufi64x2 0x23c
#define MN_vextracti64x2 0x247
#define MN_vinserti64x2 0x255
#define MN_vbroadcasti64x2 0x262
#define MN_sm3 0x272
#define MN_int3 0x276
#define MN_vmovdqa64 0x27b
#define MN_xsavec64 0x285
#define MN_xsave64 (MN_fxsave64 + 1)
#define MN_fxsave64 0x28e
#define MN_endbr64 0x297
#define MN_xrstor64 (MN_fxrstor64 + 1)
#define MN_fxrstor64 0x29f
#define MN_xsaves64 0x2a9
#define MN_xrstors64 0x2b2
#define MN_xsaveopt64 0x2bc
#define MN_vmovdqu64 0x2c7
#define MN_rex64 0x2d1
#define MN_xsha384 0x2d7
#define MN_sm4 0x2df
#define MN_sha1rnds4 0x2e3
#define MN_vsm4rnds4 0x2ed
#define MN_vshuff32x4 0x2f7
#define MN_vextractf32x4 0x302
#define MN_vinsertf32x4 0x310
#define MN_vbroadcastf32x4 0x31d
#define MN_vshufi32x4 0x32d
#define MN_vextracti32x4 0x338
#define MN_vinserti32x4 0x346
#define MN_vbroadcasti32x4 0x353
#define MN_vextractf64x4 0x363
#define MN_vinsertf64x4 0x371
#define MN_vbroadcastf64x4 0x37e
#define MN_vextracti64x4 0x38e
#define MN_vinserti64x4 0x39c
#define MN_vbroadcasti64x4 0x3a9
#define MN_vsm4key4 0x3b9
#define MN_data16 0x3c2
#define MN_vfmsub231bf16 0x3c9
#define MN_vfnmsub231bf16 0x3d7
#define MN_vfmadd231bf16 0x3e6
#define MN_vfnmadd231bf16 0x3f4
#define MN_vfmsub132bf16 0x403
#define MN_vfnmsub132bf16 0x411
#define MN_vfmadd132bf16 0x420
#define MN_vfnmadd132bf16 0x42e
#define MN_vcvtne2ps2bf16 0x43d
#define MN_vcvtneps2bf16 0x44c
#define MN_vfmsub213bf16 0x45a
#define MN_vfnmsub213bf16 0x468
#define MN_vfmadd213bf16 0x477
#define MN_vfnmadd213bf16 0x485
#define MN_vsubbf16 0x494
#define MN_vaddbf16 0x49d
#define MN_vreducebf16 0x4a6
#define MN_vrndscalebf16 0x4b2
#define MN_vscalefbf16 0x4c0
#define MN_vmulbf16 0x4cc
#define MN_vminbf16 0x4d5
#define MN_vrcpbf16 0x4de
#define MN_vcmpbf16 0x4e7
#define MN_vgetexpbf16 0x4f0
#define MN_vcomisbf16 0x4fc
#define MN_vfpclassbf16 0x507
#define MN_vgetmantbf16 0x514
#define MN_vrsqrtbf16 0x521
#define MN_vsqrtbf16 0x52c
#define MN_vdivbf16 0x536
#define MN_vminmaxbf16 0x53f
#define MN_vmaxbf16 0x54b
#define MN_tconjtfp16 0x554
#define MN_addr16 0x55f
#define MN_vmovdqu16 0x566
#define MN_vbmacor16x16x16 0x570
#define MN_vbmacxor16x16x16 0x580
#define MN_xsha256 0x591
#define MN_encodekey256 0x599
#define MN_vperm2f128 0x5a6
#define MN_vextractf128 0x5b1
#define MN_vinsertf128 0x5be
#define MN_vbroadcastf128 0x5ca
#define MN_vperm2i128 0x5d9
#define MN_vextracti128 0x5e4
#define MN_vinserti128 0x5f1
#define MN_vbroadcasti128 0x5fd
#define MN_encodekey128 0x60c
#define MN_vcvt2ph2bf8 0x619
#define MN_vcvtbiasph2bf8 0x625
#define MN_vcvtph2bf8 0x634
#define MN_vcvt2ph2hf8 0x63f
#define MN_vcvtbiasph2hf8 0x64b
#define MN_vcvtph2hf8 0x65a
#define MN_vmovdqu8 0x665
#define MN_vextractf32x8 0x66e
#define MN_vinsertf32x8 0x67c
#define MN_vbroadcastf32x8 0x689
#define MN_vextracti32x8 0x699
#define MN_vinserti32x8 0x6a7
#define MN_vbroadcasti32x8 0x6b4
#define MN_ud2a 0x6c4
#define MN_aaa 0x6c9
#define MN_daa 0x6cd
#define MN_ssca 0x6d1
#define MN_lea 0x6d6
#define MN_invlpga 0x6da
#define MN_pusha 0x6e2
#define MN_ja 0x6e8
#define MN_jna 0x6eb
#define MN_ccmpna 0x6ef
#define MN_setna 0x6f6
#define MN_ctestna 0x6fc
#define MN_setzuna 0x704
#define MN_cmovna (MN_fcmovna + 1)
#define MN_fcmovna (MN_cfcmovna + 1)
#define MN_cfcmovna 0x70c
#define MN_ccmpa 0x715
#define MN_popa 0x71b
#define MN_movntdqa (MN_vmovntdqa + 1)
#define MN_vmovntdqa 0x720
#define MN_movdqa (MN_vmovdqa + 1)
#define MN_vmovdqa 0x72a
#define MN_seta 0x732
#define MN_prefetchnta 0x737
#define MN_ctesta 0x743
#define MN_setzua 0x74a
#define MN_cmova (MN_fcmova + 1)
#define MN_fcmova (MN_cfcmova + 1)
#define MN_cfcmova 0x751
#define MN_rex_b 0x759
#define MN_ud2b 0x75f
#define MN_vpermi2b 0x764
#define MN_vpmovm2b 0x76d
#define MN_vpermt2b 0x776
#define MN_movdir64b 0x77f
#define MN_cmpxchg16b 0x789
#define MN_cmpxchg8b 0x794
#define MN_vpshab 0x79e
#define MN_sbb 0x7a5
#define MN_psubb (MN_vpsubb + 1)
#define MN_vpsubb 0x7a9
#define MN_xcrypt_ecb 0x7b0
#define MN_xcryptecb 0x7bb
#define MN_llwpcb 0x7c5
#define MN_slwpcb 0x7cc
#define MN_kaddb 0x7d3
#define MN_paddb (MN_vpaddb + 1)
#define MN_vpaddb 0x7d9
#define MN_kandb 0x7e0
#define MN_vpexpandb 0x7e6
#define MN_vpmovusdb 0x7f0
#define MN_vpmovsdb 0x7fa
#define MN_udb 0x803
#define MN_vpmovdb 0x807
#define MN_vpcomgeb 0x80f
#define MN_vpcomleb 0x818
#define MN_vpcmpnleb 0x821
#define MN_vpcmpleb 0x82b
#define MN_vpcomfalseb 0x834
#define MN_vpcomtrueb 0x840
#define MN_xcrypt_cfb 0x84b
#define MN_xcryptcfb 0x856
#define MN_xcrypt_ofb 0x860
#define MN_xcryptofb 0x86b
#define MN_pshufb (MN_vpshufb + 1)
#define MN_vpshufb 0x875
#define MN_invlpgb 0x87d
#define MN_pavgb (MN_vpavgb + 1)
#define MN_vpavgb 0x885
#define MN_jb 0x88c
#define MN_pbndkb 0x88f
#define MN_pmovmskb (MN_vpmovmskb + 1)
#define MN_vpmovmskb 0x896
#define MN_vpshlb 0x8a0
#define MN_kshiftlb 0x8a7
#define MN_gf2p8mulb (MN_vgf2p8mulb + 1)
#define MN_vgf2p8mulb 0x8b0
#define MN_vpblendmb 0x8bb
#define MN_vptestnmb 0x8c5
#define MN_vpcomb 0x8cf
#define MN_vpshufbitqmb 0x8d6
#define MN_vpermb 0x8e3
#define MN_vptestmb 0x8ea
#define MN_kandnb 0x8f3
#define MN_psignb (MN_vpsignb + 1)
#define MN_vpsignb 0x8fa
#define MN_jnb 0x902
#define MN_ccmpnb 0x906
#define MN_setnb 0x90d
#define MN_ctestnb 0x913
#define MN_setzunb 0x91b
#define MN_cmovnb (MN_fcmovnb + 1)
#define MN_fcmovnb (MN_cfcmovnb + 1)
#define MN_cfcmovnb 0x923
#define MN_ccmpb 0x92c
#define MN_vpcmpb 0x932
#define MN_vpcomeqb 0x939
#define MN_gf2p8affineqb (MN_vgf2p8affineqb + 1)
#define MN_vgf2p8affineqb 0x942
#define MN_vpcomneqb 0x951
#define MN_vpcmpneqb 0x95b
#define MN_pcmpeqb (MN_vpcmpeqb + 1)
#define MN_vpcmpeqb 0x965
#define MN_vpmovusqb 0x96e
#define MN_vpmovsqb 0x978
#define MN_vpmultishiftqb 0x981
#define MN_gf2p8affineinvqb (MN_vgf2p8affineinvqb + 1)
#define MN_vgf2p8affineinvqb 0x990
#define MN_vpmovqb 0x9a2
#define MN_rex_rb 0x9aa
#define MN_korb 0x9b1
#define MN_kxnorb 0x9b6
#define MN_kxorb 0x9bd
#define MN_pinsrb (MN_vpinsrb + 1)
#define MN_vpinsrb 0x9c3
#define MN_kshiftrb 0x9cb
#define MN_pextrb (MN_vpextrb + 1)
#define MN_vpextrb 0x9d4
#define MN_rex_wrb 0x9dc
#define MN_pabsb (MN_vpabsb + 1)
#define MN_vpabsb 0x9e4
#define MN_psubsb (MN_vpsubsb + 1)
#define MN_vpsubsb 0x9eb
#define MN_paddsb (MN_vpaddsb + 1)
#define MN_vpaddsb 0x9f3
#define MN_pminsb (MN_vpminsb + 1)
#define MN_vpminsb 0x9fb
#define MN_vmovrsb 0xa03
#define MN_vpcompressb 0xa0b
#define MN_psubusb (MN_vpsubusb + 1)
#define MN_vpsubusb 0xa17
#define MN_paddusb (MN_vpaddusb + 1)
#define MN_vpaddusb 0xa20
#define MN_pavgusb 0xa29
#define MN_movsb 0xa31
#define MN_pmaxsb (MN_vpmaxsb + 1)
#define MN_vpmaxsb 0xa37
#define MN_setb 0xa3f
#define MN_vpcomgtb 0xa44
#define MN_pcmpgtb (MN_vpcmpgtb + 1)
#define MN_vpcmpgtb 0xa4d
#define MN_vpcomltb 0xa56
#define MN_vpcmpnltb 0xa5f
#define MN_vpcmpltb 0xa69
#define MN_vpopcntb 0xa72
#define MN_knotb 0xa7b
#define MN_vprotb 0xa81
#define MN_vpbroadcastb 0xa88
#define MN_ctestb 0xa95
#define MN_ktestb 0xa9c
#define MN_kortestb 0xaa3
#define MN_vpcomgeub 0xaac
#define MN_vpcomleub 0xab6
#define MN_vpcmpnleub 0xac0
#define MN_vpcmpleub 0xacb
#define MN_vpcomfalseub 0xad5
#define MN_vpcomtrueub 0xae2
#define MN_vpcomub 0xaee
#define MN_pminub (MN_vpminub + 1)
#define MN_vpminub 0xaf6
#define MN_vpcmpub 0xafe
#define MN_vpcomequb 0xb06
#define MN_vpcomnequb 0xb10
#define MN_vpcmpnequb 0xb1b
#define MN_vpcmpequb 0xb26
#define MN_sub (MN_fsub + 1)
#define MN_fsub (MN_pfsub + 1)
#define MN_pfsub 0xb30
#define MN_fisub 0xb36
#define MN_vpcomgtub 0xb3c
#define MN_vpcomltub 0xb46
#define MN_vpcmpnltub 0xb50
#define MN_vpcmpltub 0xb5b
#define MN_pmaxub (MN_vpmaxub + 1)
#define MN_vpmaxub 0xb65
#define MN_setzub 0xb6d
#define MN_pblendvb (MN_vpblendvb + 1)
#define MN_vpblendvb 0xb74
#define MN_vbitrevb 0xb7e
#define MN_cmovb (MN_fcmovb + 1)
#define MN_fcmovb (MN_cfcmovb + 1)
#define MN_cfcmovb 0xb87
#define MN_kmovb 0xb8f
#define MN_rex_wb 0xb95
#define MN_clwb 0xb9c
#define MN_packsswb (MN_vpacksswb + 1)
#define MN_vpacksswb 0xba1
#define MN_packuswb (MN_vpackuswb + 1)
#define MN_vpackuswb 0xbab
#define MN_vpmovuswb 0xbb5
#define MN_vpmovswb 0xbbf
#define MN_vpmovwb 0xbc8
#define MN_rex_xb 0xbd0
#define MN_rex_rxb 0xbd7
#define MN_rex_wrxb 0xbdf
#define MN_rex_wxb 0xbe8
#define MN_movzb 0xbf0
#define MN_clac 0xbf6
#define MN_stac 0xbfb
#define MN_xcrypt_cbc 0xc00
#define MN_xcryptcbc 0xc0b
#define MN_pfacc 0xc15
#define MN_pfnacc 0xc1b
#define MN_pfpnacc 0xc22
#define MN_adc 0xc2a
#define MN_dec (MN_aesdec + 3)
#define MN_aesdec (MN_vaesdec + 1)
#define MN_vaesdec 0xc2e
#define MN_getsec 0xc36
#define MN_xsavec 0xc3d
#define MN_blcic 0xc44
#define MN_blsic 0xc4a
#define MN_jc 0xc50
#define MN_t1mskc 0xc53
#define MN_salc 0xc5a
#define MN_clc 0xc5f
#define MN_cmc 0xc63
#define MN_aesimc (MN_vaesimc + 1)
#define MN_vaesimc 0xc67
#define MN_rdpmc 0xc6f
#define MN_aesenc (MN_vaesenc + 1)
#define MN_vaesenc 0xc75
#define MN_inc 0xc7d
#define MN_jnc 0xc81
#define MN_ccmpnc 0xc85
#define MN_setnc 0xc8c
#define MN_ctestnc 0xc92
#define MN_vmfunc 0xc9a
#define MN_setzunc 0xca1
#define MN_cmovnc (MN_cfcmovnc + 2)
#define MN_cfcmovnc 0xca9
#define MN_tlbsync 0xcb2
#define MN_ccmpc 0xcba
#define MN_rdtsc 0xcc0
#define MN_btc 0xcc6
#define MN_setc 0xcca
#define MN_stc (MN_ctestc + 3)
#define MN_ctestc 0xccf
#define MN_setzuc 0xcd6
#define MN_cmovc (MN_cfcmovc + 2)
#define MN_cfcmovc 0xcdd
#define MN_vpermi2d 0xce5
#define MN_vpmovm2d 0xcee
#define MN_vpermt2d 0xcf7
#define MN_vpbroadcastmw2d 0xd00
#define MN_aad 0xd10
#define MN_vmread 0xd14
#define MN_rmpread 0xd1b
#define MN_vpshad 0xd23
#define MN_vmload 0xd2a
#define MN_psrad (MN_vpsrad + 1)
#define MN_vpsrad 0xd31
#define MN_vphaddbd 0xd38
#define MN_vphaddubd 0xd41
#define MN_phsubd (MN_vphsubd + 1)
#define MN_vphsubd 0xd4b
#define MN_psubd (MN_vpsubd + 1)
#define MN_vpsubd 0xd53
#define MN_pmovsxbd (MN_vpmovsxbd + 1)
#define MN_vpmovsxbd 0xd5a
#define MN_pmovzxbd (MN_vpmovzxbd + 1)
#define MN_vpmovzxbd 0xd64
#define MN_add (MN_aadd + 1)
#define MN_aadd 0xd6e
#define MN_fadd (MN_pfadd + 1)
#define MN_pfadd 0xd73
#define MN_fiadd 0xd79
#define MN_tileloadd 0xd7f
#define MN_xadd (MN_cmpnaxadd + 5)
#define MN_cmpnaxadd 0xd89
#define MN_cmpaxadd 0xd93
#define MN_cmpnbxadd 0xd9c
#define MN_cmpbxadd 0xda6
#define MN_cmpncxadd 0xdaf
#define MN_cmpcxadd 0xdb9
#define MN_cmpnaexadd 0xdc2
#define MN_cmpaexadd 0xdcd
#define MN_cmpnbexadd 0xdd7
#define MN_cmpbexadd 0xde2
#define MN_cmpngexadd 0xdec
#define MN_cmpgexadd 0xdf7
#define MN_cmpnlexadd 0xe01
#define MN_cmplexadd 0xe0c
#define MN_cmpnexadd 0xe16
#define MN_cmpexadd 0xe20
#define MN_cmppexadd 0xe29
#define MN_cmpngxadd 0xe33
#define MN_cmpgxadd 0xe3d
#define MN_cmpnlxadd 0xe46
#define MN_cmplxadd 0xe50
#define MN_cmpnoxadd 0xe59
#define MN_cmpoxadd 0xe63
#define MN_cmppoxadd 0xe6c
#define MN_cmpnpxadd 0xe76
#define MN_cmppxadd 0xe80
#define MN_cmpnsxadd 0xe89
#define MN_cmpsxadd 0xe93
#define MN_cmpnzxadd 0xe9c
#define MN_cmpzxadd 0xea6
#define MN_phaddd (MN_vphaddd + 1)
#define MN_vphaddd 0xeaf
#define MN_kaddd 0xeb7
#define MN_paddd (MN_vpaddd + 1)
#define MN_vpaddd 0xebd
#define MN_vpshldd 0xec4
#define MN_kandd 0xecc
#define MN_vpandd 0xed2
#define MN_vpexpandd 0xed9
#define MN_vpblendd 0xee3
#define MN_vpgatherdd 0xeec
#define MN_vpscatterdd 0xef7
#define MN_vpshrdd 0xf03
#define MN_vpmacsdd 0xf0b
#define MN_vpmacssdd 0xf14
#define MN_rdseed 0xf1e
#define MN_vpcomged 0xf25
#define MN_vpcomled 0xf2e
#define MN_vpcmpnled 0xf37
#define MN_vpcmpled 0xf41
#define MN_tilestored 0xf4a
#define MN_vpcomfalsed 0xf55
#define MN_ttransposed 0xf61
#define MN_vpcomtrued 0xf6d
#define MN_pi2fd 0xf78
#define MN_pshufd (MN_vpshufd + 1)
#define MN_vpshufd 0xf7e
#define MN_vpternlogd 0xf86
#define MN_pf2id 0xf91
#define MN_invpcid 0xf97
#define MN_rdpid 0xf9f
#define MN_invvpid 0xfa5
#define MN_cpuid 0xfad
#define MN_fbld 0xfb3
#define MN_cld 0xfb8
#define MN_fld 0xfbc
#define MN_shld (MN_vpshld + 2)
#define MN_vpshld 0xfc0
#define MN_fild 0xfc7
#define MN_pslld (MN_vpslld + 1)
#define MN_vpslld 0xfcc
#define MN_pmulld (MN_vpmulld + 1)
#define MN_vpmulld 0xfd3
#define MN_vprold 0xfdb
#define MN_psrld (MN_vpsrld + 1)
#define MN_vpsrld 0xfe2
#define MN_vmptrld 0xfe9
#define MN_kshiftld 0xff1
#define MN_enqcmd 0xffa
#define MN_vpblendmd 0x1001
#define MN_vptestnmd 0x100b
#define MN_vpcomd 0x1015
#define MN_vpermd 0x101c
#define MN_vptestmd 0x1023
#define MN_and (MN_aand + 1)
#define MN_aand 0x102c
#define MN_pand (MN_vpand + 1)
#define MN_vpand 0x1031
#define MN_rdrand 0x1037
#define MN_bnd 0x103e
#define MN_kandnd 0x1042
#define MN_vpandnd 0x1049
#define MN_xend 0x1051
#define MN_valignd 0x1056
#define MN_psignd (MN_vpsignd + 1)
#define MN_vpsignd 0x105e
#define MN_bound 0x1066
#define MN_slod 0x106c
#define MN_vfmaddsub231pd 0x1071
#define MN_vfmsub231pd 0x1080
#define MN_vfnmsub231pd 0x108c
#define MN_vfmsubadd231pd 0x1099
#define MN_vfmadd231pd 0x10a8
#define MN_vfnmadd231pd 0x10b4
#define MN_vfmaddsub132pd 0x10c1
#define MN_vfmsub132pd 0x10d0
#define MN_vfnmsub132pd 0x10dc
#define MN_vfmsubadd132pd 0x10e9
#define MN_vfmadd132pd 0x10f8
#define MN_vfnmadd132pd 0x1104
#define MN_vcvtph2pd 0x1111
#define MN_vpermi2pd 0x111b
#define MN_cvtpi2pd 0x1125
#define MN_vpermil2pd 0x112e
#define MN_vexp2pd 0x1139
#define MN_cvtdq2pd (MN_vcvtdq2pd + 1)
#define MN_vcvtdq2pd 0x1141
#define MN_vcvtudq2pd 0x114b
#define MN_vcvtqq2pd 0x1156
#define MN_vcvtuqq2pd 0x1160
#define MN_cvtps2pd (MN_vcvtps2pd + 1)
#define MN_vcvtps2pd 0x116b
#define MN_vpermt2pd 0x1175
#define MN_vfmaddsub213pd 0x117f
#define MN_vfmsub213pd 0x118e
#define MN_vfnmsub213pd 0x119a
#define MN_vfmsubadd213pd 0x11a7
#define MN_vfmadd213pd 0x11b6
#define MN_vfnmadd213pd 0x11c2
#define MN_vrcp14pd 0x11cf
#define MN_vrsqrt14pd 0x11d8
#define MN_vrcp28pd 0x11e3
#define MN_vrsqrt28pd 0x11ec
#define MN_movapd (MN_vmovapd + 1)
#define MN_vmovapd 0x11f7
#define MN_pswapd 0x11ff
#define MN_subpd (MN_addsubpd + 3)
#define MN_addsubpd (MN_vfmaddsubpd + 3)
#define MN_vfmaddsubpd 0x1206
#define MN_vaddsubpd 0x1212
#define MN_hsubpd (MN_vhsubpd + 1)
#define MN_vhsubpd 0x121c
#define MN_vfmsubpd 0x1224
#define MN_vfnmsubpd 0x122d
#define MN_vsubpd 0x1237
#define MN_vgatherpf0dpd 0x123e
#define MN_vscatterpf0dpd 0x124c
#define MN_vgatherpf1dpd 0x125b
#define MN_vscatterpf1dpd 0x1269
#define MN_addpd (MN_vfmsubaddpd + 6)
#define MN_vfmsubaddpd 0x1278
#define MN_haddpd (MN_vhaddpd + 1)
#define MN_vhaddpd 0x1284
#define MN_vfmaddpd 0x128c
#define MN_vfnmaddpd 0x1295
#define MN_vaddpd 0x129f
#define MN_andpd (MN_vexpandpd + 4)
#define MN_vexpandpd 0x12a6
#define MN_vandpd 0x12b0
#define MN_blendpd (MN_vblendpd + 1)
#define MN_vblendpd 0x12b7
#define MN_roundpd (MN_vroundpd + 1)
#define MN_vroundpd 0x12c0
#define MN_vgatherdpd 0x12c9
#define MN_vscatterdpd 0x12d4
#define MN_cmpunordpd (MN_vcmpunordpd + 1)
#define MN_vcmpunordpd 0x12e0
#define MN_cmpordpd (MN_vcmpordpd + 1)
#define MN_vcmpordpd 0x12ec
#define MN_vreducepd 0x12f6
#define MN_vrangepd 0x1300
#define MN_vcmpngepd 0x1309
#define MN_vcmpgepd 0x1313
#define MN_vrndscalepd 0x131c
#define MN_cmpnlepd (MN_vcmpnlepd + 1)
#define MN_vcmpnlepd 0x1328
#define MN_cmplepd (MN_vcmplepd + 1)
#define MN_vcmplepd 0x1332
#define MN_vcmpfalsepd 0x133b
#define MN_vcmptruepd 0x1347
#define MN_vscalefpd 0x1352
#define MN_shufpd (MN_vshufpd + 1)
#define MN_vshufpd 0x135c
#define MN_unpckhpd (MN_vunpckhpd + 1)
#define MN_vunpckhpd 0x1364
#define MN_movhpd (MN_vmovhpd + 1)
#define MN_vmovhpd 0x136e
#define MN_movmskpd (MN_vmovmskpd + 1)
#define MN_vmovmskpd 0x1376
#define MN_vpermilpd 0x1380
#define MN_unpcklpd (MN_vunpcklpd + 1)
#define MN_vunpcklpd 0x138a
#define MN_mulpd (MN_vmulpd + 1)
#define MN_vmulpd 0x1394
#define MN_movlpd (MN_vmovlpd + 1)
#define MN_vmovlpd 0x139b
#define MN_vpcmpd 0x13a3
#define MN_vblendmpd 0x13aa
#define MN_vfixupimmpd 0x13b4
#define MN_vpermpd 0x13c0
#define MN_andnpd (MN_vandnpd + 1)
#define MN_vandnpd 0x13c8
#define MN_minpd (MN_vminpd + 1)
#define MN_vminpd 0x13d0
#define MN_dppd (MN_vdppd + 1)
#define MN_vdppd 0x13d7
#define MN_cmppd (MN_vcmppd + 1)
#define MN_vcmppd 0x13dd
#define MN_vgetexppd 0x13e4
#define MN_vgatherpf0qpd 0x13ee
#define MN_vscatterpf0qpd 0x13fc
#define MN_vgatherpf1qpd 0x140b
#define MN_vscatterpf1qpd 0x1419
#define MN_vcmpunord_qpd 0x1428
#define MN_vcmpord_qpd 0x1436
#define MN_cmpneqpd (MN_vcmpneqpd + 1)
#define MN_vcmpneqpd 0x1442
#define MN_cmpeqpd (MN_vcmpeqpd + 1)
#define MN_vcmpeqpd 0x144c
#define MN_vcmpge_oqpd 0x1455
#define MN_vcmple_oqpd 0x1461
#define MN_vcmpfalse_oqpd 0x146d
#define MN_vcmpneq_oqpd 0x147c
#define MN_vcmpeq_oqpd 0x1489
#define MN_vcmpgt_oqpd 0x1495
#define MN_vcmplt_oqpd 0x14a1
#define MN_vgatherqpd 0x14ad
#define MN_vscatterqpd 0x14b8
#define MN_vcmpnge_uqpd 0x14c4
#define MN_vcmpnle_uqpd 0x14d1
#define MN_vcmptrue_uqpd 0x14de
#define MN_vcmpneq_uqpd 0x14ec
#define MN_vcmpeq_uqpd 0x14f9
#define MN_vcmpngt_uqpd 0x1505
#define MN_vcmpnlt_uqpd 0x1512
#define MN_orpd (MN_vorpd + 1)
#define MN_vorpd 0x151f
#define MN_xorpd (MN_vxorpd + 1)
#define MN_vxorpd 0x1525
#define MN_vcmpunord_spd 0x152c
#define MN_vcmpord_spd 0x153a
#define MN_vcmpge_ospd 0x1546
#define MN_vcmple_ospd 0x1552
#define MN_vcmpfalse_ospd 0x155e
#define MN_vcmpneq_ospd 0x156d
#define MN_vcmpeq_ospd 0x157a
#define MN_vcmpgt_ospd 0x1586
#define MN_vcmplt_ospd 0x1592
#define MN_vfpclasspd 0x159e
#define MN_incsspd 0x15a9
#define MN_rdsspd 0x15b1
#define MN_vcompresspd 0x15b8
#define MN_vcmpnge_uspd 0x15c4
#define MN_vcmpnle_uspd 0x15d1
#define MN_vcmptrue_uspd 0x15de
#define MN_vcmpneq_uspd 0x15ec
#define MN_vcmpeq_uspd 0x15f9
#define MN_vcmpngt_uspd 0x1605
#define MN_vcmpnlt_uspd 0x1612
#define MN_vcmpngtpd 0x161f
#define MN_vcmpgtpd 0x1629
#define MN_cmpnltpd (MN_vcmpnltpd + 1)
#define MN_vcmpnltpd 0x1632
#define MN_cmpltpd (MN_vcmpltpd + 1)
#define MN_vcmpltpd 0x163c
#define MN_vgetmantpd 0x1645
#define MN_movntpd (MN_vmovntpd + 1)
#define MN_vmovntpd 0x1650
#define MN_sqrtpd (MN_vsqrtpd + 1)
#define MN_vsqrtpd 0x1659
#define MN_vtestpd 0x1661
#define MN_movupd (MN_vmovupd + 1)
#define MN_vmovupd 0x1669
#define MN_blendvpd (MN_vblendvpd + 1)
#define MN_vblendvpd 0x1671
#define MN_divpd (MN_vdivpd + 1)
#define MN_vdivpd 0x167b
#define MN_vmaskmovpd 0x1682
#define MN_maxpd (MN_vminmaxpd + 4)
#define MN_vminmaxpd 0x168d
#define MN_vmaxpd 0x1697
#define MN_vfrczpd 0x169e
#define MN_vpcomeqd 0x16a6
#define MN_vpcomneqd 0x16af
#define MN_vpcmpneqd 0x16b9
#define MN_pcmpeqd (MN_vpcmpeqd + 1)
#define MN_vpcmpeqd 0x16c3
#define MN_vpgatherqd 0x16cc
#define MN_vpscatterqd 0x16d7
#define MN_vpmovusqd 0x16e3
#define MN_vpmovsqd 0x16ed
#define MN_vpmovqd 0x16f6
#define MN_shrd 0x16fe
#define MN_kord 0x1703
#define MN_kxnord 0x1708
#define MN_vpord 0x170f
#define MN_vprord 0x1715
#define MN_word (MN_aword + 1)
#define MN_aword 0x171c
#define MN_dword (MN_adword + 1)
#define MN_adword 0x1722
#define MN_kxord 0x1729
#define MN_vpxord 0x172f
#define MN_pinsrd (MN_vpinsrd + 1)
#define MN_vpinsrd 0x1736
#define MN_kshiftrd 0x173e
#define MN_pextrd (MN_vpextrd + 1)
#define MN_vpextrd 0x1747
#define MN_vfmsub231sd 0x174f
#define MN_vfnmsub231sd 0x175b
#define MN_vfmadd231sd 0x1768
#define MN_vfnmadd231sd 0x1774
#define MN_vfmsub132sd 0x1781
#define MN_vfnmsub132sd 0x178d
#define MN_vfmadd132sd 0x179a
#define MN_vfnmadd132sd 0x17a6
#define MN_vcvtsh2sd 0x17b3
#define MN_cvtsi2sd (MN_vcvtsi2sd + 1)
#define MN_vcvtsi2sd 0x17bd
#define MN_vcvtusi2sd 0x17c7
#define MN_cvtss2sd (MN_vcvtss2sd + 1)
#define MN_vcvtss2sd 0x17d2
#define MN_vfmsub213sd 0x17dc
#define MN_vfnmsub213sd 0x17e8
#define MN_vfmadd213sd 0x17f5
#define MN_vfnmadd213sd 0x1801
#define MN_vrcp14sd 0x180e
#define MN_vrsqrt14sd 0x1817
#define MN_vrcp28sd 0x1822
#define MN_vrsqrt28sd 0x182b
#define MN_pabsd (MN_vpabsd + 1)
#define MN_vpabsd 0x1836
#define MN_subsd (MN_vfmsubsd + 3)
#define MN_vfmsubsd 0x183d
#define MN_vfnmsubsd 0x1846
#define MN_vsubsd 0x1850
#define MN_addsd (MN_vfmaddsd + 3)
#define MN_vfmaddsd 0x1857
#define MN_vfnmaddsd 0x1860
#define MN_vaddsd 0x186a
#define MN_roundsd (MN_vroundsd + 1)
#define MN_vroundsd 0x1871
#define MN_cmpunordsd (MN_vcmpunordsd + 1)
#define MN_vcmpunordsd 0x187a
#define MN_cmpordsd (MN_vcmpordsd + 1)
#define MN_vcmpordsd 0x1886
#define MN_vreducesd 0x1890
#define MN_vrangesd 0x189a
#define MN_vcmpngesd 0x18a3
#define MN_vcmpgesd 0x18ad
#define MN_vrndscalesd 0x18b6
#define MN_cmpnlesd (MN_vcmpnlesd + 1)
#define MN_vcmpnlesd 0x18c2
#define MN_cmplesd (MN_vcmplesd + 1)
#define MN_vcmplesd 0x18cc
#define MN_vcmpfalsesd 0x18d5
#define MN_vcmptruesd 0x18e1
#define MN_vscalefsd 0x18ec
#define MN_comisd (MN_ucomisd + 1)
#define MN_ucomisd (MN_vucomisd + 1)
#define MN_vucomisd 0x18f6
#define MN_vcomisd 0x18ff
#define MN_mulsd (MN_vmulsd + 1)
#define MN_vmulsd 0x1907
#define MN_vfixupimmsd 0x190e
#define MN_minsd (MN_pminsd + 1)
#define MN_pminsd (MN_vpminsd + 1)
#define MN_vpminsd 0x191a
#define MN_vminsd 0x1922
#define MN_cmpsd (MN_vcmpsd + 1)
#define MN_vcmpsd 0x1929
#define MN_vgetexpsd 0x1930
#define MN_vcmpunord_qsd 0x193a
#define MN_vcmpord_qsd 0x1948
#define MN_cmpneqsd (MN_vcmpneqsd + 1)
#define MN_vcmpneqsd 0x1954
#define MN_cmpeqsd (MN_vcmpeqsd + 1)
#define MN_vcmpeqsd 0x195e
#define MN_vcmpge_oqsd 0x1967
#define MN_vcmple_oqsd 0x1973
#define MN_vcmpfalse_oqsd 0x197f
#define MN_vcmpneq_oqsd 0x198e
#define MN_vcmpeq_oqsd 0x199b
#define MN_vcmpgt_oqsd 0x19a7
#define MN_vcmplt_oqsd 0x19b3
#define MN_vcmpnge_uqsd 0x19bf
#define MN_vcmpnle_uqsd 0x19cc
#define MN_vcmptrue_uqsd 0x19d9
#define MN_vcmpneq_uqsd 0x19e7
#define MN_vcmpeq_uqsd 0x19f4
#define MN_vcmpngt_uqsd 0x1a00
#define MN_vcmpnlt_uqsd 0x1a0d
#define MN_vmovrsd 0x1a1a
#define MN_vcmpunord_ssd 0x1a22
#define MN_vcmpord_ssd 0x1a30
#define MN_vpdpbssd 0x1a3c
#define MN_tdpbssd 0x1a45
#define MN_vpcompressd 0x1a4d
#define MN_vcmpge_ossd 0x1a59
#define MN_vcmple_ossd 0x1a65
#define MN_vcmpfalse_ossd 0x1a71
#define MN_vcmpneq_ossd 0x1a80
#define MN_vcmpeq_ossd 0x1a8d
#define MN_vcmpgt_ossd 0x1a99
#define MN_vcmplt_ossd 0x1aa5
#define MN_wrssd 0x1ab1
#define MN_vfpclasssd 0x1ab7
#define MN_vcmpnge_ussd 0x1ac2
#define MN_vcmpnle_ussd 0x1acf
#define MN_vcmptrue_ussd 0x1adc
#define MN_vcmpneq_ussd 0x1aea
#define MN_vcmpeq_ussd 0x1af7
#define MN_vcmpngt_ussd 0x1b03
#define MN_vcmpnlt_ussd 0x1b10
#define MN_wrussd 0x1b1d
#define MN_vp4dpwssd 0x1b24
#define MN_vpdpwssd 0x1b2e
#define MN_vcmpngtsd 0x1b37
#define MN_vcmpgtsd 0x1b41
#define MN_cmpnltsd (MN_vcmpnltsd + 1)
#define MN_vcmpnltsd 0x1b4a
#define MN_cmpltsd (MN_vcmpltsd + 1)
#define MN_vcmpltsd 0x1b54
#define MN_vgetmantsd 0x1b5d
#define MN_movntsd 0x1b68
#define MN_sqrtsd (MN_vsqrtsd + 1)
#define MN_vsqrtsd 0x1b70
#define MN_vbroadcastsd 0x1b78
#define MN_vpdpbusd 0x1b85
#define MN_tdpbusd 0x1b8e
#define MN_vpdpwusd 0x1b96
#define MN_divsd (MN_vdivsd + 1)
#define MN_vdivsd 0x1b9f
#define MN_movsd (MN_vmovsd + 1)
#define MN_vmovsd 0x1ba6
#define MN_maxsd (MN_vminmaxsd + 4)
#define MN_vminmaxsd 0x1bad
#define MN_pmaxsd (MN_vpmaxsd + 1)
#define MN_vpmaxsd 0x1bb7
#define MN_vmaxsd 0x1bbf
#define MN_vucomxsd 0x1bc6
#define MN_vcomxsd 0x1bcf
#define MN_vfrczsd 0x1bd7
#define MN_vp2intersectd 0x1bdf
#define MN_vpconflictd 0x1bed
#define MN_vpcomgtd 0x1bf9
#define MN_pcmpgtd (MN_vpcmpgtd + 1)
#define MN_vpcmpgtd 0x1c02
#define MN_cltd 0x1c0b
#define MN_vpcomltd 0x1c10
#define MN_vpcmpnltd 0x1c19
#define MN_vpcmpltd 0x1c23
#define MN_vpopcntd 0x1c2c
#define MN_vplzcntd 0x1c35
#define MN_knotd 0x1c3e
#define MN_vprotd 0x1c44
#define MN_std (MN_vpbroadcastd + 9)
#define MN_vpbroadcastd 0x1c4b
#define MN_ktestd 0x1c58
#define MN_kortestd 0x1c5f
#define MN_cwtd 0x1c68
#define MN_vpcomgeud 0x1c6d
#define MN_vpcomleud 0x1c77
#define MN_vpcmpnleud 0x1c81
#define MN_vpcmpleud 0x1c8c
#define MN_vpcomfalseud 0x1c96
#define MN_vpcomtrueud 0x1ca3
#define MN_vpcomud 0x1caf
#define MN_pminud (MN_vpminud + 1)
#define MN_vpminud 0x1cb7
#define MN_vpcmpud 0x1cbf
#define MN_vpcomequd 0x1cc7
#define MN_vpcomnequd 0x1cd1
#define MN_vpcmpnequd 0x1cdc
#define MN_vpcmpequd 0x1ce7
#define MN_vpdpbsud 0x1cf1
#define MN_tdpbsud 0x1cfa
#define MN_vpdpwsud 0x1d02
#define MN_vpcomgtud 0x1d0b
#define MN_vpcomltud 0x1d15
#define MN_vpcmpnltud 0x1d1f
#define MN_vpcmpltud 0x1d2a
#define MN_vpdpbuud 0x1d34
#define MN_tdpbuud 0x1d3d
#define MN_vpdpwuud 0x1d45
#define MN_pmaxud (MN_vpmaxud + 1)
#define MN_vpmaxud 0x1d4e
#define MN_vpsravd 0x1d56
#define MN_vpshldvd 0x1d5e
#define MN_vpshrdvd 0x1d67
#define MN_vpsllvd 0x1d70
#define MN_vprolvd 0x1d78
#define MN_vpsrlvd 0x1d80
#define MN_invd (MN_wbinvd + 2)
#define MN_wbinvd 0x1d88
#define MN_wbnoinvd 0x1d8f
#define MN_movd (MN_kmovd + 1)
#define MN_kmovd (MN_vpmaskmovd + 5)
#define MN_vpmaskmovd 0x1d98
#define MN_vmovd 0x1da3
#define MN_vprorvd 0x1da9
#define MN_vphsubwd 0x1db1
#define MN_cwd 0x1dba
#define MN_vphaddwd 0x1dbe
#define MN_pmaddwd (MN_vpmaddwd + 1)
#define MN_vpmaddwd 0x1dc7
#define MN_punpckhwd (MN_vpunpckhwd + 1)
#define MN_vpunpckhwd 0x1dd0
#define MN_kunpckwd 0x1ddb
#define MN_punpcklwd (MN_vpunpcklwd + 1)
#define MN_vpunpcklwd 0x1de4
#define MN_vpmacswd 0x1def
#define MN_vpmadcswd 0x1df8
#define MN_vpmacsswd 0x1e02
#define MN_vpmadcsswd 0x1e0c
#define MN_vphadduwd 0x1e17
#define MN_pmovsxwd (MN_vpmovsxwd + 1)
#define MN_vpmovsxwd 0x1e21
#define MN_pmovzxwd (MN_vpmovzxwd + 1)
#define MN_vpmovzxwd 0x1e2b
#define MN_movsxd 0x1e35
#define MN_fldl2e 0x1e3c
#define MN_jae 0x1e43
#define MN_jnae 0x1e47
#define MN_ccmpnae 0x1e4c
#define MN_setnae 0x1e54
#define MN_ctestnae 0x1e5b
#define MN_setzunae 0x1e64
#define MN_cmovnae (MN_fcmovnae + 1)
#define MN_fcmovnae (MN_cfcmovnae + 1)
#define MN_cfcmovnae 0x1e6d
#define MN_ccmpae 0x1e77
#define MN_setae 0x1e7e
#define MN_ctestae 0x1e84
#define MN_setzuae 0x1e8c
#define MN_cmovae (MN_fcmovae + 1)
#define MN_fcmovae (MN_cfcmovae + 1)
#define MN_cfcmovae 0x1e94
#define MN_jbe 0x1e9d
#define MN_jnbe 0x1ea1
#define MN_ccmpnbe 0x1ea6
#define MN_setnbe 0x1eae
#define MN_ctestnbe 0x1eb5
#define MN_setzunbe 0x1ebe
#define MN_cmovnbe (MN_fcmovnbe + 1)
#define MN_fcmovnbe (MN_cfcmovnbe + 1)
#define MN_cfcmovnbe 0x1ec7
#define MN_ccmpbe 0x1ed1
#define MN_setbe 0x1ed8
#define MN_ctestbe 0x1ede
#define MN_setzube 0x1ee6
#define MN_movbe (MN_cmovbe + 1)
#define MN_cmovbe (MN_fcmovbe + 1)
#define MN_fcmovbe (MN_cfcmovbe + 1)
#define MN_cfcmovbe 0x1eee
#define MN_lfence 0x1ef7
#define MN_mfence 0x1efe
#define MN_sfence 0x1f05
#define MN_cwde 0x1f0c
#define MN_ffree 0x1f11
#define MN_jge 0x1f17
#define MN_jnge 0x1f1b
#define MN_ccmpnge 0x1f20
#define MN_setnge 0x1f28
#define MN_ctestnge 0x1f2f
#define MN_setzunge 0x1f38
#define MN_cmovnge (MN_cfcmovnge + 2)
#define MN_cfcmovnge 0x1f41
#define MN_ccmpge 0x1f4b
#define MN_pfcmpge 0x1f52
#define MN_setge 0x1f5a
#define MN_ctestge 0x1f60
#define MN_setzuge 0x1f68
#define MN_cmovge (MN_cfcmovge + 2)
#define MN_cfcmovge 0x1f70
#define MN_je 0x1f79
#define MN_fscale 0x1f7c
#define MN_jle 0x1f83
#define MN_jnle 0x1f87
#define MN_ccmpnle 0x1f8c
#define MN_setnle 0x1f94
#define MN_ctestnle 0x1f9b
#define MN_setzunle 0x1fa4
#define MN_cmovnle (MN_cfcmovnle + 2)
#define MN_cfcmovnle 0x1fad
#define MN_ccmple 0x1fb7
#define MN_setle 0x1fbe
#define MN_ctestle 0x1fc4
#define MN_setzule 0x1fcc
#define MN_cmovle (MN_cfcmovle + 2)
#define MN_cfcmovle 0x1fd4
#define MN_vmresume 0x1fdd
#define MN_jne 0x1fe6
#define MN_repne 0x1fea
#define MN_ccmpne 0x1ff0
#define MN_loopne 0x1ff7
#define MN_setne 0x1ffe
#define MN_ctestne 0x2004
#define MN_setzune 0x200c
#define MN_cmovne (MN_fcmovne + 1)
#define MN_fcmovne (MN_cfcmovne + 1)
#define MN_cfcmovne 0x2014
#define MN_repe 0x201d
#define MN_jpe 0x2022
#define MN_ccmpe 0x2026
#define MN_loope 0x202c
#define MN_setpe 0x2032
#define MN_setzupe 0x2038
#define MN_cmovpe (MN_cfcmovpe + 2)
#define MN_cfcmovpe 0x2040
#define MN_cdqe 0x2049
#define MN_xacquire 0x204e
#define MN_xstore 0x2057
#define MN_rdfsbase 0x205e
#define MN_wrfsbase 0x2067
#define MN_rdgsbase 0x2070
#define MN_wrgsbase 0x2079
#define MN_tilerelease 0x2082
#define MN_xrelease 0x208e
#define MN_pause (MN_tpause + 1)
#define MN_tpause 0x2097
#define MN_pvalidate 0x209e
#define MN_rmpupdate 0x20a8
#define MN_sete 0x20b2
#define MN_vmwrite 0x20b7
#define MN_ptwrite 0x20bf
#define MN_cldemote 0x20c7
#define MN_cteste 0x20d0
#define MN_sha1nexte 0x20d7
#define MN_setzue 0x20e1
#define MN_leave 0x20e8
#define MN_fsave 0x20ee
#define MN_vmsave 0x20f4
#define MN_fnsave 0x20fb
#define MN_xsave (MN_fxsave + 1)
#define MN_fxsave 0x2102
#define MN_cmove (MN_fcmove + 1)
#define MN_fcmove (MN_cfcmove + 1)
#define MN_cfcmove 0x2109
#define MN_serialize 0x2111
#define MN_vmxoff 0x211b
#define MN_lahf 0x2122
#define MN_sahf 0x2127
#define MN_pushf 0x212c
#define MN_ccmpf 0x2132
#define MN_popf 0x2138
#define MN_bsf 0x213d
#define MN_retf 0x2141
#define MN_ctestf 0x2146
#define MN_neg 0x214d
#define MN_ldtilecfg 0x2151
#define MN_sttilecfg 0x215b
#define MN_xchg (MN_cmpxchg + 3)
#define MN_cmpxchg 0x2165
#define MN_pconfig 0x216d
#define MN_jg 0x2175
#define MN_jng 0x2178
#define MN_ccmpng 0x217c
#define MN_xstore_rng 0x2183
#define MN_xstorerng 0x218e
#define MN_setng 0x2198
#define MN_ctestng 0x219e
#define MN_setzung 0x21a6
#define MN_cmovng (MN_cfcmovng + 2)
#define MN_cfcmovng 0x21ae
#define MN_invlpg 0x21b7
#define MN_ccmpg 0x21be
#define MN_setg 0x21c4
#define MN_ctestg 0x21c9
#define MN_setzug 0x21d0
#define MN_cmovg (MN_cfcmovg + 2)
#define MN_cfcmovg 0x21d7
#define MN_tcvtrowps2bf16h 0x21df
#define MN_vmlaunch 0x21ef
#define MN_prefetch 0x21f8
#define MN_fxch 0x2201
#define MN_tcvtrowps2phh 0x2206
#define MN_vfmaddsub231ph 0x2214
#define MN_vfmsub231ph 0x2223
#define MN_vfnmsub231ph 0x222f
#define MN_vfmsubadd231ph 0x223c
#define MN_vfmadd231ph 0x224b
#define MN_vfnmadd231ph 0x2257
#define MN_vfmaddsub132ph 0x2264
#define MN_vfmsub132ph 0x2273
#define MN_vfnmsub132ph 0x227f
#define MN_vfmsubadd132ph 0x228c
#define MN_vfmadd132ph 0x229b
#define MN_vfnmadd132ph 0x22a7
#define MN_vcvthf82ph 0x22b4
#define MN_vcvtpd2ph 0x22bf
#define MN_vcvtdq2ph 0x22c9
#define MN_vcvtudq2ph 0x22d3
#define MN_vcvtqq2ph 0x22de
#define MN_vcvtuqq2ph 0x22e8
#define MN_vcvtps2ph 0x22f3
#define MN_vcvtw2ph 0x22fd
#define MN_vcvtuw2ph 0x2306
#define MN_vfmaddsub213ph 0x2310
#define MN_vfmsub213ph 0x231f
#define MN_vfnmsub213ph 0x232b
#define MN_vfmsubadd213ph 0x2338
#define MN_vfmadd213ph 0x2347
#define MN_vfnmadd213ph 0x2353
#define MN_vsubph 0x2360
#define MN_vfcmaddcph 0x2367
#define MN_vfmaddcph 0x2372
#define MN_vfcmulcph 0x237c
#define MN_vfmulcph 0x2386
#define MN_vaddph 0x238f
#define MN_vcmpunordph 0x2396
#define MN_vcmpordph 0x23a2
#define MN_vreduceph 0x23ac
#define MN_vcmpngeph 0x23b6
#define MN_vcmpgeph 0x23c0
#define MN_vrndscaleph 0x23c9
#define MN_vcmpnleph 0x23d5
#define MN_vcmpleph 0x23df
#define MN_vcmpfalseph 0x23e8
#define MN_vcmptrueph 0x23f4
#define MN_vscalefph 0x23ff
#define MN_vmulph 0x2409
#define MN_vminph 0x2410
#define MN_vrcpph 0x2417
#define MN_vcmpph 0x241e
#define MN_vgetexpph 0x2425
#define MN_vcmpunord_qph 0x242f
#define MN_vcmpord_qph 0x243d
#define MN_vcmpneqph 0x2449
#define MN_vcmpeqph 0x2453
#define MN_vcmpge_oqph 0x245c
#define MN_vcmple_oqph 0x2468
#define MN_vcmpfalse_oqph 0x2474
#define MN_vcmpneq_oqph 0x2483
#define MN_vcmpeq_oqph 0x2490
#define MN_vcmpgt_oqph 0x249c
#define MN_vcmplt_oqph 0x24a8
#define MN_vcmpnge_uqph 0x24b4
#define MN_vcmpnle_uqph 0x24c1
#define MN_vcmptrue_uqph 0x24ce
#define MN_vcmpneq_uqph 0x24dc
#define MN_vcmpeq_uqph 0x24e9
#define MN_vcmpngt_uqph 0x24f5
#define MN_vcmpnlt_uqph 0x2502
#define MN_vcmpunord_sph 0x250f
#define MN_vcmpord_sph 0x251d
#define MN_vcmpge_osph 0x2529
#define MN_vcmple_osph 0x2535
#define MN_vcmpfalse_osph 0x2541
#define MN_vcmpneq_osph 0x2550
#define MN_vcmpeq_osph 0x255d
#define MN_vcmpgt_osph 0x2569
#define MN_vcmplt_osph 0x2575
#define MN_vfpclassph 0x2581
#define MN_vcmpnge_usph 0x258c
#define MN_vcmpnle_usph 0x2599
#define MN_vcmptrue_usph 0x25a6
#define MN_vcmpneq_usph 0x25b4
#define MN_vcmpeq_usph 0x25c1
#define MN_vcmpngt_usph 0x25cd
#define MN_vcmpnlt_usph 0x25da
#define MN_vcmpngtph 0x25e7
#define MN_vcmpgtph 0x25f1
#define MN_vcmpnltph 0x25fa
#define MN_vcmpltph 0x2604
#define MN_vgetmantph 0x260d
#define MN_vrsqrtph 0x2618
#define MN_vsqrtph 0x2621
#define MN_vdivph 0x2629
#define MN_vminmaxph 0x2630
#define MN_vmaxph 0x263a
#define MN_vpmacsdqh 0x2641
#define MN_vpmacssdqh 0x264b
#define MN_vfmsub231sh 0x2656
#define MN_vfnmsub231sh 0x2662
#define MN_vfmadd231sh 0x266f
#define MN_vfnmadd231sh 0x267b
#define MN_vfmsub132sh 0x2688
#define MN_vfnmsub132sh 0x2694
#define MN_vfmadd132sh 0x26a1
#define MN_vfnmadd132sh 0x26ad
#define MN_vcvtsd2sh 0x26ba
#define MN_vcvtsi2sh 0x26c4
#define MN_vcvtusi2sh 0x26ce
#define MN_vcvtss2sh 0x26d9
#define MN_vfmsub213sh 0x26e3
#define MN_vfnmsub213sh 0x26ef
#define MN_vfmadd213sh 0x26fc
#define MN_vfnmadd213sh 0x2708
#define MN_psmash 0x2715
#define MN_vsubsh 0x271c
#define MN_vfcmaddcsh 0x2723
#define MN_vfmaddcsh 0x272e
#define MN_vfcmulcsh 0x2738
#define MN_vfmulcsh 0x2742
#define MN_vaddsh 0x274b
#define MN_vcmpunordsh 0x2752
#define MN_vcmpordsh 0x275e
#define MN_vreducesh 0x2768
#define MN_vcmpngesh 0x2772
#define MN_vcmpgesh 0x277c
#define MN_vrndscalesh 0x2785
#define MN_vcmpnlesh 0x2791
#define MN_vcmplesh 0x279b
#define MN_vcmpfalsesh 0x27a4
#define MN_vcmptruesh 0x27b0
#define MN_vscalefsh 0x27bb
#define MN_vucomish 0x27c5
#define MN_vcomish 0x27ce
#define MN_vmulsh 0x27d6
#define MN_vminsh 0x27dd
#define MN_vrcpsh 0x27e4
#define MN_vcmpsh 0x27eb
#define MN_vgetexpsh 0x27f2
#define MN_vcmpunord_qsh 0x27fc
#define MN_vcmpord_qsh 0x280a
#define MN_vcmpneqsh 0x2816
#define MN_vcmpeqsh 0x2820
#define MN_vcmpge_oqsh 0x2829
#define MN_vcmple_oqsh 0x2835
#define MN_vcmpfalse_oqsh 0x2841
#define MN_vcmpneq_oqsh 0x2850
#define MN_vcmpeq_oqsh 0x285d
#define MN_vcmpgt_oqsh 0x2869
#define MN_vcmplt_oqsh 0x2875
#define MN_vcmpnge_uqsh 0x2881
#define MN_vcmpnle_uqsh 0x288e
#define MN_vcmptrue_uqsh 0x289b
#define MN_vcmpneq_uqsh 0x28a9
#define MN_vcmpeq_uqsh 0x28b6
#define MN_vcmpngt_uqsh 0x28c2
#define MN_vcmpnlt_uqsh 0x28cf
#define MN_vcmpunord_ssh 0x28dc
#define MN_vcmpord_ssh 0x28ea
#define MN_vcmpge_ossh 0x28f6
#define MN_vcmple_ossh 0x2902
#define MN_vcmpfalse_ossh 0x290e
#define MN_vcmpneq_ossh 0x291d
#define MN_vcmpeq_ossh 0x292a
#define MN_vcmpgt_ossh 0x2936
#define MN_vcmplt_ossh 0x2942
#define MN_vfpclasssh 0x294e
#define MN_vcmpnge_ussh 0x2959
#define MN_vcmpnle_ussh 0x2966
#define MN_vcmptrue_ussh 0x2973
#define MN_vcmpneq_ussh 0x2981
#define MN_vcmpeq_ussh 0x298e
#define MN_vcmpngt_ussh 0x299a
#define MN_vcmpnlt_ussh 0x29a7
#define MN_vcmpngtsh 0x29b4
#define MN_vcmpgtsh 0x29be
#define MN_vcmpnltsh 0x29c7
#define MN_vcmpltsh 0x29d1
#define MN_vgetmantsh 0x29da
#define MN_vrsqrtsh 0x29e5
#define MN_vsqrtsh 0x29ee
#define MN_clflush 0x29f6
#define MN_push 0x29fe
#define MN_vdivsh 0x2a03
#define MN_vmovsh 0x2a0a
#define MN_vminmaxsh 0x2a11
#define MN_vmaxsh 0x2a1b
#define MN_vucomxsh 0x2a22
#define MN_vcomxsh 0x2a2b
#define MN_blci 0x2a33
#define MN_clgi 0x2a38
#define MN_stgi 0x2a3d
#define MN_bzhi 0x2a42
#define MN_cli 0x2a47
#define MN_fcomi 0x2a4b
#define MN_fucomi 0x2a51
#define MN_feni 0x2a58
#define MN_fneni 0x2a5d
#define MN_cvttpd2pi 0x2a63
#define MN_cvtpd2pi 0x2a6d
#define MN_cvttps2pi 0x2a76
#define MN_cvtps2pi 0x2a80
#define MN_fldpi 0x2a89
#define MN_senduipi 0x2a8f
#define MN_fcompi 0x2a98
#define MN_fucompi 0x2a9f
#define MN_movdiri 0x2aa7
#define MN_pcmpestri (MN_vpcmpestri + 1)
#define MN_vpcmpestri 0x2aaf
#define MN_pcmpistri (MN_vpcmpistri + 1)
#define MN_vpcmpistri 0x2aba
#define MN_cvttsd2si (MN_vcvttsd2si + 1)
#define MN_vcvttsd2si 0x2ac5
#define MN_cvtsd2si (MN_vcvtsd2si + 1)
#define MN_vcvtsd2si 0x2ad0
#define MN_vcvttsh2si 0x2ada
#define MN_vcvtsh2si 0x2ae5
#define MN_cvttss2si (MN_vcvttss2si + 1)
#define MN_vcvttss2si 0x2aef
#define MN_cvtss2si (MN_vcvtss2si + 1)
#define MN_vcvtss2si 0x2afa
#define MN_fdisi 0x2b04
#define MN_fndisi 0x2b0a
#define MN_blsi 0x2b11
#define MN_vcvttsd2usi 0x2b16
#define MN_vcvtsd2usi 0x2b22
#define MN_vcvttsh2usi 0x2b2d
#define MN_vcvtsh2usi 0x2b39
#define MN_vcvttss2usi 0x2b44
#define MN_vcvtss2usi 0x2b50
#define MN_movnti 0x2b5b
#define MN_sti 0x2b62
#define MN_clui 0x2b66
#define MN_stui (MN_testui + 2)
#define MN_testui 0x2b6b
#define MN_notrack 0x2b72
#define MN_lock 0x2b7a
#define MN_bndmk 0x2b7f
#define MN_xresldtrk 0x2b85
#define MN_xsusldtrk 0x2b8f
#define MN_blcmsk 0x2b99
#define MN_blsmsk 0x2ba0
#define MN_tzmsk 0x2ba7
#define MN_tcvtrowps2bf16l 0x2bad
#define MN_sal 0x2bbd
#define MN_lwpval 0x2bc1
#define MN_bndcl 0x2bc8
#define MN_rcl 0x2bce
#define MN_tcvtrowps2phl 0x2bd2
#define MN_shl 0x2be0
#define MN_jl 0x2be4
#define MN_aesdec256kl 0x2be7
#define MN_aesenc256kl 0x2bf3
#define MN_aesdecwide256kl 0x2bff
#define MN_aesencwide256kl 0x2c0f
#define MN_aesdec128kl 0x2c1f
#define MN_aesenc128kl 0x2c2b
#define MN_aesdecwide128kl 0x2c37
#define MN_aesencwide128kl 0x2c47
#define MN_call (MN_tdcall + 2)
#define MN_tdcall 0x2c57
#define MN_lcall 0x2c5e
#define MN_seamcall 0x2c64
#define MN_vmmcall 0x2c6d
#define MN_vmcall 0x2c75
#define MN_syscall 0x2c7c
#define MN_vzeroall 0x2c84
#define MN_fildll 0x2c8d
#define MN_blcfill 0x2c94
#define MN_blsfill 0x2c9c
#define MN_fistpll 0x2ca4
#define MN_fisttpll 0x2cac
#define MN_jnl 0x2cb5
#define MN_ccmpnl 0x2cb9
#define MN_setnl 0x2cc0
#define MN_ctestnl 0x2cc6
#define MN_setzunl 0x2cce
#define MN_cmovnl (MN_cfcmovnl + 2)
#define MN_cfcmovnl 0x2cd6
#define MN_rol 0x2cdf
#define MN_ccmpl 0x2ce3
#define MN_arpl 0x2ce9
#define MN_vpmacsdql 0x2cee
#define MN_vpmacssdql 0x2cf8
#define MN_lsl 0x2d03
#define MN_movsl 0x2d07
#define MN_setl 0x2d0d
#define MN_ctestl 0x2d12
#define MN_cwtl 0x2d19
#define MN_mul (MN_fmul + 1)
#define MN_fmul (MN_pfmul + 1)
#define MN_pfmul 0x2d1e
#define MN_imul (MN_fimul + 1)
#define MN_fimul 0x2d24
#define MN_montmul 0x2d2a
#define MN_setzul 0x2d32
#define MN_cmovl (MN_cfcmovl + 2)
#define MN_cfcmovl 0x2d39
#define MN_vpmovb2m 0x2d41
#define MN_vpmovd2m 0x2d4a
#define MN_vpmovq2m 0x2d53
#define MN_vpmovw2m 0x2d5c
#define MN_aam 0x2d65
#define MN_fxam 0x2d69
#define MN_fprem 0x2d6e
#define MN_fcom 0x2d74
#define MN_ficom 0x2d79
#define MN_fucom 0x2d7f
#define MN_fsetpm 0x2d85
#define MN_fnsetpm 0x2d8c
#define MN_frstpm 0x2d94
#define MN_vpperm 0x2d9b
#define MN_pcmpestrm (MN_vpcmpestrm + 1)
#define MN_vpcmpestrm 0x2da2
#define MN_pcmpistrm (MN_vpcmpistrm + 1)
#define MN_vpcmpistrm 0x2dad
#define MN_rsm 0x2db8
#define MN_fpatan 0x2dbc
#define MN_fptan 0x2dc3
#define MN_bndcn 0x2dc9
#define MN_andn (MN_pandn + 1)
#define MN_pandn (MN_vpandn + 1)
#define MN_vpandn 0x2dcf
#define MN_in (MN_xbegin + 4)
#define MN_xbegin 0x2dd6
#define MN_pfmin 0x2ddd
#define MN_fsin 0x2de3
#define MN_vmxon 0x2de8
#define MN_vmrun 0x2dee
#define MN_jo 0x2df4
#define MN_jno 0x2df7
#define MN_ccmpno 0x2dfb
#define MN_setno 0x2e02
#define MN_ctestno 0x2e08
#define MN_setzuno 0x2e10
#define MN_cmovno (MN_cfcmovno + 2)
#define MN_cfcmovno 0x2e18
#define MN_jpo 0x2e21
#define MN_ccmpo 0x2e25
#define MN_setpo 0x2e2b
#define MN_setzupo 0x2e31
#define MN_cmovpo (MN_cfcmovpo + 2)
#define MN_cfcmovpo 0x2e39
#define MN_cqo 0x2e42
#define MN_tilezero 0x2e46
#define MN_clzero 0x2e4f
#define MN_seto 0x2e56
#define MN_into 0x2e5b
#define MN_cqto 0x2e60
#define MN_ctesto 0x2e65
#define MN_ssto 0x2e6c
#define MN_setzuo 0x2e71
#define MN_cmovo (MN_cfcmovo + 2)
#define MN_cfcmovo 0x2e78
#define MN_push2p 0x2e80
#define MN_pop2p 0x2e87
#define MN_bswap 0x2e8d
#define MN_fsubp 0x2e93
#define MN_pfrcp 0x2e99
#define MN_rdtscp 0x2e9f
#define MN_faddp 0x2ea6
#define MN_pdep 0x2eac
#define MN_ffreep 0x2eb1
#define MN_rep 0x2eb8
#define MN_pushp 0x2ebc
#define MN_fcomip 0x2ec2
#define MN_fucomip 0x2ec9
#define MN_jp 0x2ed1
#define MN_fmulp 0x2ed4
#define MN_cmp (MN_scmp + 1)
#define MN_scmp 0x2eda
#define MN_jmp (MN_ljmp + 1)
#define MN_ljmp 0x2edf
#define MN_fcomp 0x2ee4
#define MN_ficomp 0x2eea
#define MN_fucomp 0x2ef1
#define MN_jnp 0x2ef8
#define MN_setnp 0x2efc
#define MN_setzunp 0x2f02
#define MN_cmovnp (MN_cfcmovnp + 2)
#define MN_cfcmovnp 0x2f0a
#define MN_nop (MN_fnop + 1)
#define MN_fnop 0x2f13
#define MN_loop 0x2f18
#define MN_pop 0x2f1d
#define MN_fcompp 0x2f21
#define MN_fucompp 0x2f28
#define MN_popp 0x2f30
#define MN_fsubrp 0x2f35
#define MN_fdivrp 0x2f3c
#define MN_rstorssp 0x2f43
#define MN_saveprevssp 0x2f4c
#define MN_setp 0x2f58
#define MN_fbstp 0x2f5d
#define MN_fdecstp 0x2f63
#define MN_fincstp 0x2f6b
#define MN_fstp 0x2f73
#define MN_fistp 0x2f78
#define MN_fisttp 0x2f7e
#define MN_movddup (MN_vmovddup + 1)
#define MN_vmovddup 0x2f85
#define MN_movshdup (MN_vmovshdup + 1)
#define MN_vmovshdup 0x2f8e
#define MN_movsldup (MN_vmovsldup + 1)
#define MN_vmovsldup 0x2f98
#define MN_setzup 0x2fa2
#define MN_fdivp 0x2fa9
#define MN_cmovp (MN_cfcmovp + 2)
#define MN_cfcmovp 0x2faf
#define MN_xmodexp 0x2fb7
#define MN_vpbroadcastmb2q 0x2fbf
#define MN_vpermi2q 0x2fcf
#define MN_vpmovm2q 0x2fd8
#define MN_movdq2q 0x2fe1
#define MN_vpermt2q 0x2fe9
#define MN_vpshaq 0x2ff2
#define MN_vpsraq 0x2ff9
#define MN_vphaddbq 0x3000
#define MN_vphaddubq 0x3009
#define MN_psubq (MN_vpsubq + 1)
#define MN_vpsubq 0x3013
#define MN_pmovsxbq (MN_vpmovsxbq + 1)
#define MN_vpmovsxbq 0x301a
#define MN_pmovzxbq (MN_vpmovzxbq + 1)
#define MN_vpmovzxbq 0x3024
#define MN_cvttpd2dq (MN_vcvttpd2dq + 1)
#define MN_vcvttpd2dq 0x302e
#define MN_cvtpd2dq (MN_vcvtpd2dq + 1)
#define MN_vcvtpd2dq 0x3039
#define MN_vcvttph2dq 0x3043
#define MN_vcvtph2dq 0x304e
#define MN_movq2dq 0x3058
#define MN_cvttps2dq (MN_vcvttps2dq + 1)
#define MN_vcvttps2dq 0x3060
#define MN_cvtps2dq (MN_vcvtps2dq + 1)
#define MN_vcvtps2dq 0x306b
#define MN_vphsubdq 0x3075
#define MN_cdq 0x307e
#define MN_kaddq 0x3082
#define MN_paddq (MN_vpaddq + 1)
#define MN_vpaddq 0x3088
#define MN_vphadddq 0x308f
#define MN_punpckhdq (MN_vpunpckhdq + 1)
#define MN_vpunpckhdq 0x3098
#define MN_kunpckdq 0x30a3
#define MN_vpshldq 0x30ac
#define MN_punpckldq (MN_vpunpckldq + 1)
#define MN_vpunpckldq 0x30b4
#define MN_pslldq (MN_vpslldq + 1)
#define MN_vpslldq 0x30bf
#define MN_psrldq (MN_vpsrldq + 1)
#define MN_vpsrldq 0x30c7
#define MN_pmuldq (MN_vpmuldq + 1)
#define MN_vpmuldq 0x30cf
#define MN_kandq 0x30d7
#define MN_vpandq 0x30dd
#define MN_vpexpandq 0x30e4
#define MN_punpckhqdq (MN_vpunpckhqdq + 1)
#define MN_vpunpckhqdq 0x30ee
#define MN_pclmulhqhqdq (MN_vpclmulhqhqdq + 1)
#define MN_vpclmulhqhqdq 0x30fa
#define MN_pclmullqhqdq (MN_vpclmullqhqdq + 1)
#define MN_vpclmullqhqdq 0x3108
#define MN_punpcklqdq (MN_vpunpcklqdq + 1)
#define MN_vpunpcklqdq 0x3116
#define MN_pclmulhqlqdq (MN_vpclmulhqlqdq + 1)
#define MN_vpclmulhqlqdq 0x3122
#define MN_pclmullqlqdq (MN_vpclmullqlqdq + 1)
#define MN_vpclmullqlqdq 0x3130
#define MN_pclmulqdq (MN_vpclmulqdq + 1)
#define MN_vpclmulqdq 0x313e
#define MN_vpgatherdq 0x3149
#define MN_vpscatterdq 0x3154
#define MN_vpshrdq 0x3160
#define MN_movntdq (MN_vmovntdq + 1)
#define MN_vmovntdq 0x3168
#define MN_vcvttpd2udq 0x3171
#define MN_vcvtpd2udq 0x317d
#define MN_vcvttph2udq 0x3188
#define MN_vcvtph2udq 0x3194
#define MN_vcvttps2udq 0x319f
#define MN_vcvtps2udq 0x31ab
#define MN_vphaddudq 0x31b6
#define MN_pmuludq (MN_vpmuludq + 1)
#define MN_vpmuludq 0x31c0
#define MN_pmovsxdq (MN_vpmovsxdq + 1)
#define MN_vpmovsxdq 0x31c9
#define MN_pmovzxdq (MN_vpmovzxdq + 1)
#define MN_vpmovzxdq 0x31d3
#define MN_vpcomgeq 0x31dd
#define MN_vpcomleq 0x31e6
#define MN_vpcmpnleq 0x31ef
#define MN_vpcmpleq 0x31f9
#define MN_pfcmpeq 0x3202
#define MN_vpcomfalseq 0x320a
#define MN_vpcomtrueq 0x3216
#define MN_vpternlogq 0x3221
#define MN_vpshlq 0x322c
#define MN_psllq (MN_vpsllq + 1)
#define MN_vpsllq 0x3233
#define MN_vpmullq 0x323a
#define MN_vprolq 0x3242
#define MN_psrlq (MN_vpsrlq + 1)
#define MN_vpsrlq 0x3249
#define MN_kshiftlq 0x3250
#define MN_vpblendmq 0x3259
#define MN_vptestnmq 0x3263
#define MN_vpcomq 0x326d
#define MN_vpermq 0x3274
#define MN_vptestmq 0x327b
#define MN_kandnq 0x3284
#define MN_vpandnq 0x328b
#define MN_valignq 0x3293
#define MN_vpcmpq 0x329b
#define MN_incsspq 0x32a2
#define MN_rdsspq 0x32aa
#define MN_vcvttpd2qq 0x32b1
#define MN_vcvtpd2qq 0x32bc
#define MN_vcvttph2qq 0x32c6
#define MN_vcvtph2qq 0x32d1
#define MN_vcvttps2qq 0x32db
#define MN_vcvtps2qq 0x32e6
#define MN_vpcomeqq 0x32f0
#define MN_vpcomneqq 0x32f9
#define MN_vpcmpneqq 0x3303
#define MN_pcmpeqq (MN_vpcmpeqq + 1)
#define MN_vpcmpeqq 0x330d
#define MN_vpgatherqq 0x3316
#define MN_vpscatterqq 0x3321
#define MN_vcvttpd2uqq 0x332d
#define MN_vcvtpd2uqq 0x3339
#define MN_vcvttph2uqq 0x3344
#define MN_vcvtph2uqq 0x3350
#define MN_vcvttps2uqq 0x335b
#define MN_vcvtps2uqq 0x3367
#define MN_korq 0x3372
#define MN_kxnorq 0x3377
#define MN_vporq 0x337e
#define MN_vprorq 0x3384
#define MN_kxorq 0x338b
#define MN_vpxorq 0x3391
#define MN_pinsrq (MN_vpinsrq + 1)
#define MN_vpinsrq 0x3398
#define MN_kshiftrq 0x33a0
#define MN_extrq (MN_pextrq + 1)
#define MN_pextrq (MN_vpextrq + 1)
#define MN_vpextrq 0x33a9
#define MN_vpabsq 0x33b1
#define MN_vpminsq 0x33b8
#define MN_vmovrsq 0x33c0
#define MN_vpcompressq 0x33c8
#define MN_wrssq 0x33d4
#define MN_wrussq 0x33da
#define MN_vpmaxsq 0x33e1
#define MN_vp2intersectq 0x33e9
#define MN_vpconflictq 0x33f7
#define MN_vpcomgtq 0x3403
#define MN_pcmpgtq (MN_vpcmpgtq + 1)
#define MN_vpcmpgtq 0x340c
#define MN_cltq 0x3415
#define MN_vpcomltq 0x341a
#define MN_vpcmpnltq 0x3423
#define MN_vpcmpltq 0x342d
#define MN_vpopcntq 0x3436
#define MN_vplzcntq 0x343f
#define MN_movntq 0x3448
#define MN_knotq 0x344f
#define MN_vprotq 0x3455
#define MN_insertq 0x345c
#define MN_vpbroadcastq 0x3464
#define MN_ktestq 0x3471
#define MN_kortestq 0x3478
#define MN_vpcomgeuq 0x3481
#define MN_vpcomleuq 0x348b
#define MN_vpcmpnleuq 0x3495
#define MN_vpcmpleuq 0x34a0
#define MN_vpcomfalseuq 0x34aa
#define MN_vpcomtrueuq 0x34b7
#define MN_vpmadd52huq 0x34c3
#define MN_vpmadd52luq 0x34cf
#define MN_vpcomuq 0x34db
#define MN_vpminuq 0x34e3
#define MN_vpcmpuq 0x34eb
#define MN_vpcomequq 0x34f3
#define MN_vpcomnequq 0x34fd
#define MN_vpcmpnequq 0x3508
#define MN_vpcmpequq 0x3513
#define MN_vpcomgtuq 0x351d
#define MN_vpcomltuq 0x3527
#define MN_vpcmpnltuq 0x3531
#define MN_vpcmpltuq 0x353c
#define MN_vpmaxuq 0x3546
#define MN_vpsravq 0x354e
#define MN_vpshldvq 0x3556
#define MN_vpshrdvq 0x355f
#define MN_vpsllvq 0x3568
#define MN_vprolvq 0x3570
#define MN_vpsrlvq 0x3578
#define MN_movq (MN_kmovq + 1)
#define MN_kmovq (MN_maskmovq + 3)
#define MN_maskmovq (MN_vpmaskmovq + 2)
#define MN_vpmaskmovq 0x3580
#define MN_vmovq 0x358b
#define MN_vprorvq 0x3591
#define MN_vphaddwq 0x3599
#define MN_vphadduwq 0x35a2
#define MN_pmovsxwq (MN_vpmovsxwq + 1)
#define MN_vpmovsxwq 0x35ac
#define MN_pmovzxwq (MN_vpmovzxwq + 1)
#define MN_vpmovzxwq 0x35b6
#define MN_rex_r 0x35c0
#define MN_vmclear 0x35c6
#define MN_lar 0x35ce
#define MN_sar 0x35d2
#define MN_fsubr (MN_pfsubr + 1)
#define MN_pfsubr 0x35d6
#define MN_fisubr 0x35dd
#define MN_rcr 0x35e4
#define MN_vzeroupper 0x35e8
#define MN_enter (MN_sysenter + 3)
#define MN_sysenter 0x35f3
#define MN_shr 0x35fc
#define MN_clr 0x3600
#define MN_palignr (MN_vpalignr + 1)
#define MN_vpalignr 0x3604
#define MN_or (MN_aor + 1)
#define MN_aor 0x360d
#define MN_por (MN_vpor + 1)
#define MN_vpor 0x3611
#define MN_ror 0x3616
#define MN_monitor (MN_umonitor + 1)
#define MN_umonitor 0x361a
#define MN_frstor 0x3623
#define MN_xrstor (MN_fxrstor + 1)
#define MN_fxrstor 0x362a
#define MN_xor (MN_axor + 1)
#define MN_axor 0x3632
#define MN_pxor (MN_vpxor + 1)
#define MN_vpxor 0x3637
#define MN_verr 0x363d
#define MN_bsr 0x3642
#define MN_ldmxcsr (MN_vldmxcsr + 1)
#define MN_vldmxcsr 0x3646
#define MN_stmxcsr (MN_vstmxcsr + 1)
#define MN_vstmxcsr 0x364f
#define MN_blsr 0x3658
#define MN_rdmsr (MN_urdmsr + 1)
#define MN_urdmsr 0x365d
#define MN_wrmsr (MN_uwrmsr + 1)
#define MN_uwrmsr 0x3664
#define MN_btr 0x366b
#define MN_xcrypt_ctr 0x366f
#define MN_xcryptctr 0x367a
#define MN_ltr 0x3684
#define MN_str 0x3688
#define MN_bextr 0x368c
#define MN_fdivr 0x3692
#define MN_fidivr 0x3698
#define MN_rex_wr 0x369f
#define MN_vcvt2ph2bf8s 0x36a6
#define MN_vcvtbiasph2bf8s 0x36b3
#define MN_vcvtph2bf8s 0x36c3
#define MN_vcvt2ph2hf8s 0x36cf
#define MN_vcvtbiasph2hf8s 0x36dc
#define MN_vcvtph2hf8s 0x36ec
#define MN_aas 0x36f8
#define MN_scas 0x36fc
#define MN_das 0x3701
#define MN_fabs 0x3705
#define MN_jmpabs 0x370a
#define MN_movabs 0x3711
#define MN_vcvttbf162ibs 0x3718
#define MN_vcvtbf162ibs 0x3726
#define MN_vcvttph2ibs 0x3733
#define MN_vcvtph2ibs 0x373f
#define MN_vcvttps2ibs 0x374a
#define MN_vcvtps2ibs 0x3756
#define MN_vcvttbf162iubs 0x3761
#define MN_vcvtbf162iubs 0x3770
#define MN_vcvttph2iubs 0x377e
#define MN_vcvtph2iubs 0x378b
#define MN_vcvttps2iubs 0x3797
#define MN_vcvtps2iubs 0x37a4
#define MN_cs (MN_blcs + 2)
#define MN_blcs 0x37b0
#define MN_ds (MN_lds + 1)
#define MN_lds 0x37b5
#define MN_enqcmds 0x37b9
#define MN_lods 0x37c1
#define MN_vpdpbssds 0x37c6
#define MN_vp4dpwssds 0x37d0
#define MN_vpdpwssds 0x37db
#define MN_vpdpbusds 0x37e5
#define MN_vpdpwusds 0x37ef
#define MN_vpdpbsuds 0x37f9
#define MN_vpdpwsuds 0x3803
#define MN_vpdpbuuds 0x380d
#define MN_vpdpwuuds 0x3817
#define MN_es (MN_les + 1)
#define MN_les 0x3821
#define MN_xsaves 0x3825
#define MN_fs (MN_lfs + 1)
#define MN_lfs 0x382c
#define MN_gs (MN_lkgs + 2)
#define MN_lkgs 0x3830
#define MN_lgs 0x3835
#define MN_swapgs 0x3839
#define MN_fchs 0x3840
#define MN_vcvttsd2sis 0x3845
#define MN_vcvttss2sis 0x3851
#define MN_vcvttsd2usis 0x385d
#define MN_vcvttss2usis 0x386a
#define MN_js 0x3877
#define MN_encls 0x387a
#define MN_emms (MN_femms + 1)
#define MN_femms 0x3880
#define MN_ins (MN_lwpins + 3)
#define MN_lwpins 0x3886
#define MN_jns 0x388d
#define MN_ccmpns 0x3891
#define MN_wrmsrns 0x3898
#define MN_setns 0x38a0
#define MN_ctestns 0x38a6
#define MN_setzuns 0x38ae
#define MN_cmovns (MN_cfcmovns + 2)
#define MN_cfcmovns 0x38b6
#define MN_fcos 0x38bf
#define MN_fsincos 0x38c4
#define MN_stos 0x38cc
#define MN_vfmaddsub231ps 0x38d1
#define MN_vfmsub231ps 0x38e0
#define MN_vfnmsub231ps 0x38ec
#define MN_vfmsubadd231ps 0x38f9
#define MN_vfmadd231ps 0x3908
#define MN_vfnmadd231ps 0x3914
#define MN_vfmaddsub132ps 0x3921
#define MN_vfmsub132ps 0x3930
#define MN_vfnmsub132ps 0x393c
#define MN_vfmsubadd132ps 0x3949
#define MN_vfmadd132ps 0x3958
#define MN_vfnmadd132ps 0x3964
#define MN_tmmultf32ps (MN_ttmmultf32ps + 1)
#define MN_ttmmultf32ps 0x3971
#define MN_vcvtneebf162ps 0x397e
#define MN_vbcstnebf162ps 0x398d
#define MN_vcvtneobf162ps 0x399c
#define MN_cvtpd2ps (MN_vcvtpd2ps + 1)
#define MN_vcvtpd2ps 0x39ab
#define MN_tcvtrowd2ps 0x39b5
#define MN_vcvtneeph2ps 0x39c1
#define MN_vcvtneoph2ps 0x39ce
#define MN_vcvtph2ps 0x39db
#define MN_vbcstnesh2ps 0x39e5
#define MN_vpermi2ps 0x39f2
#define MN_cvtpi2ps 0x39fc
#define MN_vpermil2ps 0x3a05
#define MN_vexp2ps 0x3a10
#define MN_cvtdq2ps (MN_vcvtdq2ps + 1)
#define MN_vcvtdq2ps 0x3a18
#define MN_vcvtudq2ps 0x3a22
#define MN_vcvtqq2ps 0x3a2d
#define MN_vcvtuqq2ps 0x3a37
#define MN_vpermt2ps 0x3a42
#define MN_vfmaddsub213ps 0x3a4c
#define MN_vfmsub213ps 0x3a5b
#define MN_vfnmsub213ps 0x3a67
#define MN_vfmsubadd213ps 0x3a74
#define MN_vfmadd213ps 0x3a83
#define MN_vfnmadd213ps 0x3a8f
#define MN_vrcp14ps 0x3a9c
#define MN_vrsqrt14ps 0x3aa5
#define MN_tdpbf16ps (MN_ttdpbf16ps + 1)
#define MN_ttdpbf16ps 0x3ab0
#define MN_vdpbf16ps 0x3abb
#define MN_tcmmrlfp16ps (MN_ttcmmrlfp16ps + 1)
#define MN_ttcmmrlfp16ps 0x3ac5
#define MN_tcmmimfp16ps (MN_tconjtcmmimfp16ps + 5)
#define MN_tconjtcmmimfp16ps 0x3ad3
#define MN_ttcmmimfp16ps 0x3ae5
#define MN_tdpfp16ps (MN_ttdpfp16ps + 1)
#define MN_ttdpfp16ps 0x3af3
#define MN_vrcp28ps 0x3afe
#define MN_vrsqrt28ps 0x3b07
#define MN_tdphbf8ps 0x3b12
#define MN_tdpbf8ps 0x3b1c
#define MN_tdpbhf8ps 0x3b25
#define MN_tdphf8ps 0x3b2f
#define MN_movaps (MN_vmovaps + 1)
#define MN_vmovaps 0x3b38
#define MN_subps (MN_addsubps + 3)
#define MN_addsubps (MN_vfmaddsubps + 3)
#define MN_vfmaddsubps 0x3b40
#define MN_vaddsubps 0x3b4c
#define MN_hsubps (MN_vhsubps + 1)
#define MN_vhsubps 0x3b56
#define MN_vfmsubps 0x3b5e
#define MN_vfnmsubps 0x3b67
#define MN_vsubps 0x3b71
#define MN_vgatherpf0dps 0x3b78
#define MN_vscatterpf0dps 0x3b86
#define MN_vgatherpf1dps 0x3b95
#define MN_vscatterpf1dps 0x3ba3
#define MN_addps (MN_vfmsubaddps + 6)
#define MN_vfmsubaddps 0x3bb2
#define MN_haddps (MN_vhaddps + 1)
#define MN_vhaddps 0x3bbe
#define MN_v4fmaddps 0x3bc6
#define MN_vfmaddps 0x3bd0
#define MN_v4fnmaddps 0x3bd9
#define MN_vfnmaddps 0x3be4
#define MN_vaddps 0x3bee
#define MN_andps (MN_vexpandps + 4)
#define MN_vexpandps 0x3bf5
#define MN_vandps 0x3bff
#define MN_blendps (MN_vblendps + 1)
#define MN_vblendps 0x3c06
#define MN_roundps (MN_vroundps + 1)
#define MN_vroundps 0x3c0f
#define MN_vgatherdps 0x3c18
#define MN_vscatterdps 0x3c23
#define MN_cmpunordps (MN_vcmpunordps + 1)
#define MN_vcmpunordps 0x3c2f
#define MN_cmpordps (MN_vcmpordps + 1)
#define MN_vcmpordps 0x3c3b
#define MN_vreduceps 0x3c45
#define MN_vrangeps 0x3c4f
#define MN_vcmpngeps 0x3c58
#define MN_vcmpgeps 0x3c62
#define MN_vrndscaleps 0x3c6b
#define MN_cmpnleps (MN_vcmpnleps + 1)
#define MN_vcmpnleps 0x3c77
#define MN_cmpleps (MN_vcmpleps + 1)
#define MN_vcmpleps 0x3c81
#define MN_vcmpfalseps 0x3c8a
#define MN_vcmptrueps 0x3c96
#define MN_vscalefps 0x3ca1
#define MN_shufps (MN_vshufps + 1)
#define MN_vshufps 0x3cab
#define MN_unpckhps (MN_vunpckhps + 1)
#define MN_vunpckhps 0x3cb3
#define MN_movlhps (MN_vmovlhps + 1)
#define MN_vmovlhps 0x3cbd
#define MN_vdpphps 0x3cc6
#define MN_movhps (MN_vmovhps + 1)
#define MN_vmovhps 0x3cce
#define MN_movmskps (MN_vmovmskps + 1)
#define MN_vmovmskps 0x3cd6
#define MN_movhlps (MN_vmovhlps + 1)
#define MN_vmovhlps 0x3ce0
#define MN_vpermilps 0x3ce9
#define MN_unpcklps (MN_vunpcklps + 1)
#define MN_vunpcklps 0x3cf3
#define MN_mulps (MN_vmulps + 1)
#define MN_vmulps 0x3cfd
#define MN_movlps (MN_vmovlps + 1)
#define MN_vmovlps 0x3d04
#define MN_cmps (MN_ccmps + 1)
#define MN_ccmps 0x3d0c
#define MN_vblendmps 0x3d12
#define MN_vfixupimmps 0x3d1c
#define MN_vpermps 0x3d28
#define MN_andnps (MN_vandnps + 1)
#define MN_vandnps 0x3d30
#define MN_minps (MN_vminps + 1)
#define MN_vminps 0x3d38
#define MN_seamops 0x3d3f
#define MN_rcpps (MN_vrcpps + 1)
#define MN_vrcpps 0x3d47
#define MN_dpps (MN_vdpps + 1)
#define MN_vdpps 0x3d4e
#define MN_cmpps (MN_vcmpps + 1)
#define MN_vcmpps 0x3d54
#define MN_vgetexpps 0x3d5b
#define MN_vgatherpf0qps 0x3d65
#define MN_vscatterpf0qps 0x3d73
#define MN_vgatherpf1qps 0x3d82
#define MN_vscatterpf1qps 0x3d90
#define MN_vcmpunord_qps 0x3d9f
#define MN_vcmpord_qps 0x3dad
#define MN_cmpneqps (MN_vcmpneqps + 1)
#define MN_vcmpneqps 0x3db9
#define MN_cmpeqps (MN_vcmpeqps + 1)
#define MN_vcmpeqps 0x3dc3
#define MN_vcmpge_oqps 0x3dcc
#define MN_vcmple_oqps 0x3dd8
#define MN_vcmpfalse_oqps 0x3de4
#define MN_vcmpneq_oqps 0x3df3
#define MN_vcmpeq_oqps 0x3e00
#define MN_vcmpgt_oqps 0x3e0c
#define MN_vcmplt_oqps 0x3e18
#define MN_vgatherqps 0x3e24
#define MN_vscatterqps 0x3e2f
#define MN_vcmpnge_uqps 0x3e3b
#define MN_vcmpnle_uqps 0x3e48
#define MN_vcmptrue_uqps 0x3e55
#define MN_vcmpneq_uqps 0x3e63
#define MN_vcmpeq_uqps 0x3e70
#define MN_vcmpngt_uqps 0x3e7c
#define MN_vcmpnlt_uqps 0x3e89
#define MN_orps (MN_vorps + 1)
#define MN_vorps 0x3e96
#define MN_xorps (MN_vxorps + 1)
#define MN_vxorps 0x3e9c
#define MN_vcmpunord_sps 0x3ea3
#define MN_vcmpord_sps 0x3eb1
#define MN_vcmpge_osps 0x3ebd
#define MN_vcmple_osps 0x3ec9
#define MN_vcmpfalse_osps 0x3ed5
#define MN_vcmpneq_osps 0x3ee4
#define MN_vcmpeq_osps 0x3ef1
#define MN_vcmpgt_osps 0x3efd
#define MN_vcmplt_osps 0x3f09
#define MN_vfpclassps 0x3f15
#define MN_vcompressps 0x3f20
#define MN_vcmpnge_usps 0x3f2c
#define MN_vcmpnle_usps 0x3f39
#define MN_vcmptrue_usps 0x3f46
#define MN_vcmpneq_usps 0x3f54
#define MN_vcmpeq_usps 0x3f61
#define MN_vcmpngt_usps 0x3f6d
#define MN_vcmpnlt_usps 0x3f7a
#define MN_extractps (MN_vextractps + 1)
#define MN_vextractps 0x3f87
#define MN_vcmpngtps 0x3f92
#define MN_vcmpgtps 0x3f9c
#define MN_cmpnltps (MN_vcmpnltps + 1)
#define MN_vcmpnltps 0x3fa5
#define MN_cmpltps (MN_vcmpltps + 1)
#define MN_vcmpltps 0x3faf
#define MN_vgetmantps 0x3fb8
#define MN_movntps (MN_vmovntps + 1)
#define MN_vmovntps 0x3fc3
#define MN_insertps (MN_vinsertps + 1)
#define MN_vinsertps 0x3fcc
#define MN_sqrtps (MN_rsqrtps + 1)
#define MN_rsqrtps (MN_vrsqrtps + 1)
#define MN_vrsqrtps 0x3fd6
#define MN_vsqrtps 0x3fdf
#define MN_vtestps 0x3fe7
#define MN_movups (MN_vmovups + 1)
#define MN_vmovups 0x3fef
#define MN_blendvps (MN_vblendvps + 1)
#define MN_vblendvps 0x3ff7
#define MN_divps (MN_vdivps + 1)
#define MN_vdivps 0x4001
#define MN_vmaskmovps 0x4008
#define MN_maxps (MN_vminmaxps + 4)
#define MN_vminmaxps 0x4013
#define MN_vmaxps 0x401d
#define MN_vfrczps 0x4024
#define MN_vcvttpd2dqs 0x402c
#define MN_vcvttps2dqs 0x4038
#define MN_vcvttpd2udqs 0x4044
#define MN_vcvttps2udqs 0x4051
#define MN_vcvttpd2qqs 0x405e
#define MN_vcvttps2qqs 0x406a
#define MN_vcvttpd2uqqs 0x4076
#define MN_vcvttps2uqqs 0x4083
#define MN_t2rpntlvwz0rs 0x4090
#define MN_t2rpntlvwz1rs 0x409e
#define MN_tileloaddrs 0x40ac
#define MN_xrstors 0x40b8
#define MN_movrs 0x40c0
#define MN_ss (MN_vfmsub231ss + 9)
#define MN_vfmsub231ss 0x40c6
#define MN_vfnmsub231ss 0x40d2
#define MN_vfmadd231ss 0x40df
#define MN_vfnmadd231ss 0x40eb
#define MN_vfmsub132ss 0x40f8
#define MN_vfnmsub132ss 0x4104
#define MN_vfmadd132ss 0x4111
#define MN_vfnmadd132ss 0x411d
#define MN_cvtsd2ss (MN_vcvtsd2ss + 1)
#define MN_vcvtsd2ss 0x412a
#define MN_vcvtsh2ss 0x4134
#define MN_cvtsi2ss (MN_vcvtsi2ss + 1)
#define MN_vcvtsi2ss 0x413e
#define MN_vcvtusi2ss 0x4148
#define MN_vfmsub213ss 0x4153
#define MN_vfnmsub213ss 0x415f
#define MN_vfmadd213ss 0x416c
#define MN_vfnmadd213ss 0x4178
#define MN_vrcp14ss 0x4185
#define MN_vrsqrt14ss 0x418e
#define MN_vrcp28ss 0x4199
#define MN_vrsqrt28ss 0x41a2
#define MN_subss (MN_vfmsubss + 3)
#define MN_vfmsubss 0x41ad
#define MN_vfnmsubss 0x41b6
#define MN_vsubss 0x41c0
#define MN_addss (MN_v4fmaddss + 4)
#define MN_v4fmaddss 0x41c7
#define MN_vfmaddss 0x41d1
#define MN_v4fnmaddss 0x41da
#define MN_vfnmaddss 0x41e5
#define MN_vaddss 0x41ef
#define MN_roundss (MN_vroundss + 1)
#define MN_vroundss 0x41f6
#define MN_cmpunordss (MN_vcmpunordss + 1)
#define MN_vcmpunordss 0x41ff
#define MN_cmpordss (MN_vcmpordss + 1)
#define MN_vcmpordss 0x420b
#define MN_vreducess 0x4215
#define MN_vrangess 0x421f
#define MN_vcmpngess 0x4228
#define MN_vcmpgess 0x4232
#define MN_vrndscaless 0x423b
#define MN_cmpnless (MN_vcmpnless + 1)
#define MN_vcmpnless 0x4247
#define MN_cmpless (MN_vcmpless + 1)
#define MN_vcmpless 0x4251
#define MN_vcmpfalsess 0x425a
#define MN_vcmptruess 0x4266
#define MN_vscalefss 0x4271
#define MN_comiss (MN_ucomiss + 1)
#define MN_ucomiss (MN_vucomiss + 1)
#define MN_vucomiss 0x427b
#define MN_vcomiss 0x4284
#define MN_lss (MN_mulss + 2)
#define MN_mulss (MN_vmulss + 1)
#define MN_vmulss 0x428c
#define MN_vfixupimmss 0x4293
#define MN_minss (MN_vminss + 1)
#define MN_vminss 0x429f
#define MN_rcpss (MN_vrcpss + 1)
#define MN_vrcpss 0x42a6
#define MN_cmpss (MN_vcmpss + 1)
#define MN_vcmpss 0x42ad
#define MN_vgetexpss 0x42b4
#define MN_vcmpunord_qss 0x42be
#define MN_vcmpord_qss 0x42cc
#define MN_cmpneqss (MN_vcmpneqss + 1)
#define MN_vcmpneqss 0x42d8
#define MN_cmpeqss (MN_vcmpeqss + 1)
#define MN_vcmpeqss 0x42e2
#define MN_vcmpge_oqss 0x42eb
#define MN_vcmple_oqss 0x42f7
#define MN_vcmpfalse_oqss 0x4303
#define MN_vcmpneq_oqss 0x4312
#define MN_vcmpeq_oqss 0x431f
#define MN_vcmpgt_oqss 0x432b
#define MN_vcmplt_oqss 0x4337
#define MN_vcmpnge_uqss 0x4343
#define MN_vcmpnle_uqss 0x4350
#define MN_vcmptrue_uqss 0x435d
#define MN_vcmpneq_uqss 0x436b
#define MN_vcmpeq_uqss 0x4378
#define MN_vcmpngt_uqss 0x4384
#define MN_vcmpnlt_uqss 0x4391
#define MN_vcmpunord_sss 0x439e
#define MN_vcmpord_sss 0x43ac
#define MN_vcmpge_osss 0x43b8
#define MN_vcmple_osss 0x43c4
#define MN_vcmpfalse_osss 0x43d0
#define MN_vcmpneq_osss 0x43df
#define MN_vcmpeq_osss 0x43ec
#define MN_vcmpgt_osss 0x43f8
#define MN_vcmplt_osss 0x4404
#define MN_vfpclassss 0x4410
#define MN_vcmpnge_usss 0x441b
#define MN_vcmpnle_usss 0x4428
#define MN_vcmptrue_usss 0x4435
#define MN_vcmpneq_usss 0x4443
#define MN_vcmpeq_usss 0x4450
#define MN_vcmpngt_usss 0x445c
#define MN_vcmpnlt_usss 0x4469
#define MN_vcmpngtss 0x4476
#define MN_vcmpgtss 0x4480
#define MN_cmpnltss (MN_vcmpnltss + 1)
#define MN_vcmpnltss 0x4489
#define MN_cmpltss (MN_vcmpltss + 1)
#define MN_vcmpltss 0x4493
#define MN_vgetmantss 0x449c
#define MN_movntss 0x44a7
#define MN_sqrtss (MN_rsqrtss + 1)
#define MN_rsqrtss (MN_vrsqrtss + 1)
#define MN_vrsqrtss 0x44af
#define MN_vsqrtss 0x44b8
#define MN_vbroadcastss 0x44c0
#define MN_divss (MN_vdivss + 1)
#define MN_vdivss 0x44cd
#define MN_movss (MN_vmovss + 1)
#define MN_vmovss 0x44d4
#define MN_maxss (MN_vminmaxss + 4)
#define MN_vminmaxss 0x44db
#define MN_vmaxss 0x44e5
#define MN_vucomxss 0x44ec
#define MN_vcomxss 0x44f5
#define MN_vfrczss 0x44fd
#define MN_bts 0x4505
#define MN_erets 0x4509
#define MN_sets 0x450f
#define MN_clts 0x4514
#define MN_ctests 0x4519
#define MN_outs 0x4520
#define MN_setzus 0x4525
#define MN_movs (MN_cmovs + 1)
#define MN_cmovs (MN_cfcmovs + 2)
#define MN_cfcmovs 0x452c
#define MN_fldl2t 0x4534
#define MN_xlat 0x453b
#define MN_bt 0x4540
#define MN_fxtract 0x4543
#define MN_lgdt 0x454b
#define MN_sgdt 0x4550
#define MN_lidt 0x4555
#define MN_sidt 0x455a
#define MN_fldt 0x455f
#define MN_lldt 0x4564
#define MN_sldt 0x4569
#define MN_ret (MN_iret + 1)
#define MN_iret (MN_uiret + 1)
#define MN_uiret 0x456e
#define MN_lret 0x4574
#define MN_seamret 0x4579
#define MN_sysret 0x4581
#define MN_hreset 0x4588
#define MN_pfcmpgt 0x458f
#define MN_ht 0x4597
#define MN_wait (MN_fwait + 1)
#define MN_fwait 0x459a
#define MN_mwait (MN_umwait + 1)
#define MN_umwait 0x45a0
#define MN_mcommit 0x45a7
#define MN_finit 0x45af
#define MN_skinit 0x45b5
#define MN_fninit 0x45bc
#define MN_vmgexit 0x45c3
#define MN_sysexit 0x45cb
#define MN_hlt 0x45d3
#define MN_popcnt 0x45d7
#define MN_lzcnt 0x45de
#define MN_tzcnt 0x45e4
#define MN_hnt 0x45ea
#define MN_int (MN_frndint + 4)
#define MN_frndint 0x45ee
#define MN_not 0x45f6
#define MN_invept 0x45fa
#define MN_ccmpt 0x4601
#define MN_xsaveopt 0x4607
#define MN_clflushopt 0x4610
#define MN_fstpt 0x461b
#define MN_xabort 0x4621
#define MN_fsqrt 0x4628
#define MN_pfrsqrt 0x462e
#define MN_aesdeclast (MN_vaesdeclast + 1)
#define MN_vaesdeclast 0x4636
#define MN_aesenclast (MN_vaesenclast + 1)
#define MN_vaesenclast 0x4642
#define MN_test (MN_ptest + 1)
#define MN_ptest (MN_vptest + 1)
#define MN_vptest 0x464e
#define MN_xtest 0x4655
#define MN_fst 0x465b
#define MN_fist 0x465f
#define MN_rdmsrlist 0x4664
#define MN_wrmsrlist 0x466e
#define MN_aeskeygenassist (MN_vaeskeygenassist + 1)
#define MN_vaeskeygenassist 0x4678
#define MN_vmptrst 0x4689
#define MN_ftst 0x4691
#define MN_rmpadjust 0x4696
#define MN_ctestt 0x46a0
#define MN_out 0x46a7
#define MN_pext 0x46ab
#define MN_bndcu 0x46b0
#define MN_enclu 0x46b6
#define MN_fcmovnu 0x46bc
#define MN_lddqu (MN_vlddqu + 1)
#define MN_vlddqu 0x46c4
#define MN_movdqu (MN_maskmovdqu + 4)
#define MN_maskmovdqu (MN_vmaskmovdqu + 1)
#define MN_vmaskmovdqu 0x46cb
#define MN_vmovdqu 0x46d7
#define MN_rdpkru 0x46df
#define MN_wrpkru 0x46e6
#define MN_rdpru 0x46ed
#define MN_eretu 0x46f3
#define MN_fcmovu 0x46f9
#define MN_imulzu 0x4700
#define MN_xgetbv 0x4707
#define MN_xsetbv 0x470e
#define MN_div (MN_fdiv + 1)
#define MN_fdiv 0x4715
#define MN_idiv (MN_fidiv + 1)
#define MN_fidiv 0x471a
#define MN_enclv 0x4720
#define MN_fldenv 0x4726
#define MN_fstenv 0x472d
#define MN_fnstenv 0x4734
#define MN_mov (MN_vpcmov + 3)
#define MN_vpcmov 0x473c
#define MN_bndmov 0x4743
#define MN_smov 0x474a
#define MN_rex_w 0x474f
#define MN_vcvttph2w 0x4755
#define MN_vcvtph2w 0x475f
#define MN_vpermi2w 0x4768
#define MN_vpmovm2w 0x4771
#define MN_vpermt2w 0x477a
#define MN_vpshaw 0x4783
#define MN_psraw (MN_vpsraw + 1)
#define MN_vpsraw 0x478a
#define MN_vphsubbw 0x4791
#define MN_cbw 0x479a
#define MN_psadbw (MN_vdbpsadbw + 3)
#define MN_vdbpsadbw 0x479e
#define MN_mpsadbw (MN_vmpsadbw + 1)
#define MN_vmpsadbw 0x47a8
#define MN_vpsadbw 0x47b1
#define MN_vphaddbw 0x47b9
#define MN_punpckhbw (MN_vpunpckhbw + 1)
#define MN_vpunpckhbw 0x47c2
#define MN_kunpckbw 0x47cd
#define MN_punpcklbw (MN_vpunpcklbw + 1)
#define MN_vpunpcklbw 0x47d6
#define MN_vphaddubw 0x47e1
#define MN_phsubw (MN_vphsubw + 1)
#define MN_vphsubw 0x47eb
#define MN_psubw (MN_vpsubw + 1)
#define MN_vpsubw 0x47f3
#define MN_pmovsxbw (MN_vpmovsxbw + 1)
#define MN_vpmovsxbw 0x47fa
#define MN_pmovzxbw (MN_vpmovzxbw + 1)
#define MN_vpmovzxbw 0x4804
#define MN_fldcw 0x480e
#define MN_fstcw 0x4814
#define MN_fnstcw 0x481a
#define MN_phaddw (MN_vphaddw + 1)
#define MN_vphaddw 0x4821
#define MN_kaddw 0x4829
#define MN_paddw (MN_vpaddw + 1)
#define MN_vpaddw 0x482f
#define MN_vpshldw 0x4836
#define MN_kandw 0x483e
#define MN_vpexpandw 0x4844
#define MN_pblendw (MN_vpblendw + 1)
#define MN_vpblendw 0x484e
#define MN_vpshrdw 0x4857
#define MN_packssdw (MN_vpackssdw + 1)
#define MN_vpackssdw 0x485f
#define MN_packusdw (MN_vpackusdw + 1)
#define MN_vpackusdw 0x4869
#define MN_vpmovusdw 0x4873
#define MN_vpmovsdw 0x487d
#define MN_vpmovdw 0x4886
#define MN_vpcomgew 0x488e
#define MN_vpcomlew 0x4897
#define MN_vpcmpnlew 0x48a0
#define MN_vpcmplew 0x48aa
#define MN_vpcomfalsew 0x48b3
#define MN_vpcomtruew 0x48bf
#define MN_pi2fw 0x48ca
#define MN_pshufw 0x48d0
#define MN_pavgw (MN_vpavgw + 1)
#define MN_vpavgw 0x48d7
#define MN_prefetchw 0x48de
#define MN_pshufhw (MN_vpshufhw + 1)
#define MN_vpshufhw 0x48e8
#define MN_pmulhw (MN_vpmulhw + 1)
#define MN_vpmulhw 0x48f1
#define MN_pf2iw 0x48f9
#define MN_pshuflw (MN_vpshuflw + 1)
#define MN_vpshuflw 0x48ff
#define MN_vpshlw 0x4908
#define MN_psllw (MN_vpsllw + 1)
#define MN_vpsllw 0x490f
#define MN_pmullw (MN_vpmullw + 1)
#define MN_vpmullw 0x4916
#define MN_psrlw (MN_vpsrlw + 1)
#define MN_vpsrlw 0x491e
#define MN_kshiftlw 0x4925
#define MN_vpblendmw 0x492e
#define MN_vptestnmw 0x4938
#define MN_vpcomw 0x4942
#define MN_vpermw 0x4949
#define MN_vptestmw 0x4950
#define MN_kandnw 0x4959
#define MN_psignw (MN_vpsignw + 1)
#define MN_vpsignw 0x4960
#define MN_tilemovrow 0x4968
#define MN_vpcmpw 0x4973
#define MN_vpcomeqw 0x497a
#define MN_vpcomneqw 0x4983
#define MN_vpcmpneqw 0x498d
#define MN_pcmpeqw (MN_vpcmpeqw + 1)
#define MN_vpcmpeqw 0x4997
#define MN_vpmovusqw 0x49a0
#define MN_vpmovsqw 0x49aa
#define MN_vpmovqw 0x49b3
#define MN_verw 0x49bb
#define MN_pmulhrw 0x49c0
#define MN_korw 0x49c8
#define MN_kxnorw 0x49cd
#define MN_kxorw 0x49d4
#define MN_pinsrw (MN_vpinsrw + 1)
#define MN_vpinsrw 0x49da
#define MN_kshiftrw 0x49e2
#define MN_pextrw (MN_vpextrw + 1)
#define MN_vpextrw 0x49eb
#define MN_pabsw (MN_vpabsw + 1)
#define MN_vpabsw 0x49f3
#define MN_pmaddubsw (MN_vpmaddubsw + 1)
#define MN_vpmaddubsw 0x49fa
#define MN_phsubsw (MN_vphsubsw + 1)
#define MN_vphsubsw 0x4a05
#define MN_psubsw (MN_vpsubsw + 1)
#define MN_vpsubsw 0x4a0e
#define MN_phaddsw (MN_vphaddsw + 1)
#define MN_vphaddsw 0x4a16
#define MN_paddsw (MN_vpaddsw + 1)
#define MN_vpaddsw 0x4a1f
#define MN_lmsw 0x4a27
#define MN_smsw 0x4a2c
#define MN_pminsw (MN_vpminsw + 1)
#define MN_vpminsw 0x4a31
#define MN_pmulhrsw (MN_vpmulhrsw + 1)
#define MN_vpmulhrsw 0x4a39
#define MN_vmovrsw 0x4a43
#define MN_vpcompressw 0x4a4b
#define MN_fstsw 0x4a57
#define MN_fnstsw 0x4a5d
#define MN_psubusw (MN_vpsubusw + 1)
#define MN_vpsubusw 0x4a64
#define MN_paddusw (MN_vpaddusw + 1)
#define MN_vpaddusw 0x4a6d
#define MN_movsw 0x4a76
#define MN_pmaxsw (MN_vpmaxsw + 1)
#define MN_vpmaxsw 0x4a7c
#define MN_cbtw 0x4a84
#define MN_vpcomgtw 0x4a89
#define MN_pcmpgtw (MN_vpcmpgtw + 1)
#define MN_vpcmpgtw 0x4a92
#define MN_vpcomltw 0x4a9b
#define MN_vpcmpnltw 0x4aa4
#define MN_vpcmpltw 0x4aae
#define MN_vpopcntw 0x4ab7
#define MN_knotw 0x4ac0
#define MN_vprotw 0x4ac6
#define MN_vpbroadcastw 0x4acd
#define MN_ktestw 0x4ada
#define MN_kortestw 0x4ae1
#define MN_vcvttph2uw 0x4aea
#define MN_vcvtph2uw 0x4af5
#define MN_vpcomgeuw 0x4aff
#define MN_vpcomleuw 0x4b09
#define MN_vpcmpnleuw 0x4b13
#define MN_vpcmpleuw 0x4b1e
#define MN_vpcomfalseuw 0x4b28
#define MN_vpcomtrueuw 0x4b35
#define MN_pmulhuw (MN_vpmulhuw + 1)
#define MN_vpmulhuw 0x4b41
#define MN_vpcomuw 0x4b4a
#define MN_pminuw (MN_vpminuw + 1)
#define MN_vpminuw 0x4b52
#define MN_vpcmpuw 0x4b5a
#define MN_vpcomequw 0x4b62
#define MN_vpcomnequw 0x4b6c
#define MN_vpcmpnequw 0x4b77
#define MN_vpcmpequw 0x4b82
#define MN_phminposuw (MN_vphminposuw + 1)
#define MN_vphminposuw 0x4b8c
#define MN_vpcomgtuw 0x4b98
#define MN_vpcomltuw 0x4ba2
#define MN_vpcmpnltuw 0x4bac
#define MN_vpcmpltuw 0x4bb7
#define MN_pmaxuw (MN_vpmaxuw + 1)
#define MN_vpmaxuw 0x4bc1
#define MN_vpsravw 0x4bc9
#define MN_vpshldvw 0x4bd1
#define MN_vpshrdvw 0x4bda
#define MN_vpsllvw 0x4be3
#define MN_vpsrlvw 0x4beb
#define MN_kmovw 0x4bf3
#define MN_vmovw 0x4bf9
#define MN_vpmacsww 0x4bff
#define MN_vpmacssww 0x4c08
#define MN_movzw 0x4c12
#define MN_rex_x 0x4c18
#define MN_fyl2x 0x4c1e
#define MN_rex64x 0x4c24
#define MN_vcvtneps2bf16x 0x4c2b
#define MN_vfpclassbf16x 0x4c3a
#define MN_vcvtph2bf8x 0x4c48
#define MN_vcvtph2hf8x 0x4c54
#define MN_pfmax 0x4c60
#define MN_adcx 0x4c66
#define MN_bndldx 0x4c6b
#define MN_vfpclasspdx 0x4c72
#define MN_fclex 0x4c7e
#define MN_fnclex 0x4c84
#define MN_rex 0x4c8b
#define MN_vcvtpd2phx 0x4c8f
#define MN_vcvtdq2phx 0x4c9a
#define MN_vcvtudq2phx 0x4ca5
#define MN_vcvtqq2phx 0x4cb1
#define MN_vcvtuqq2phx 0x4cbc
#define MN_vcvt2ps2phx 0x4cc8
#define MN_vcvtps2phx 0x4cd4
#define MN_vfpclassphx 0x4cdf
#define MN_shlx 0x4ceb
#define MN_mulx 0x4cf0
#define MN_adox 0x4cf5
#define MN_vcvttpd2dqx 0x4cfa
#define MN_vcvtpd2dqx 0x4d06
#define MN_vcvttpd2udqx 0x4d11
#define MN_vcvtpd2udqx 0x4d1e
#define MN_rex_rx 0x4d2a
#define MN_sarx 0x4d31
#define MN_shrx 0x4d36
#define MN_rorx 0x4d3b
#define MN_monitorx 0x4d40
#define MN_rex_wrx 0x4d49
#define MN_vcvtph2bf8sx 0x4d51
#define MN_vcvtph2hf8sx 0x4d5e
#define MN_vcvtpd2psx 0x4d6b
#define MN_vcvtph2psx 0x4d76
#define MN_vcvtqq2psx 0x4d81
#define MN_vcvtuqq2psx 0x4d8c
#define MN_vfpclasspsx 0x4d98
#define MN_vcvttpd2dqsx 0x4da4
#define MN_vcvttpd2udqsx 0x4db1
#define MN_movsx 0x4dbf
#define MN_mwaitx 0x4dc5
#define MN_bndstx 0x4dcc
#define MN_rex_wx 0x4dd3
#define MN_rexx 0x4dda
#define MN_vcvtps2phxx 0x4ddf
#define MN_movzx 0x4deb
#define MN_rex64y 0x4df1
#define MN_vcvtneps2bf16y 0x4df8
#define MN_vfpclassbf16y 0x4e07
#define MN_vcvtph2bf8y 0x4e15
#define MN_vcvtph2hf8y 0x4e21
#define MN_vfpclasspdy 0x4e2d
#define MN_loadiwkey 0x4e39
#define MN_vcvtpd2phy 0x4e43
#define MN_vcvtdq2phy 0x4e4e
#define MN_vcvtudq2phy 0x4e59
#define MN_vcvtqq2phy 0x4e65
#define MN_vcvtuqq2phy 0x4e70
#define MN_vfpclassphy 0x4e7c
#define MN_vcvttpd2dqy 0x4e88
#define MN_vcvtpd2dqy 0x4e94
#define MN_vcvttpd2udqy 0x4e9f
#define MN_vcvtpd2udqy 0x4eac
#define MN_rmpquery 0x4eb8
#define MN_vcvtph2bf8sy 0x4ec1
#define MN_vcvtph2hf8sy 0x4ece
#define MN_clrssbsy 0x4edb
#define MN_setssbsy 0x4ee4
#define MN_vcvtpd2psy 0x4eed
#define MN_vcvtqq2psy 0x4ef8
#define MN_vcvtuqq2psy 0x4f03
#define MN_vfpclasspsy 0x4f0f
#define MN_vcvttpd2dqsy 0x4f1b
#define MN_vcvttpd2udqsy 0x4f28
#define MN_rex64xy 0x4f36
#define MN_rexy 0x4f3e
#define MN_vcvtps2phxy 0x4f43
#define MN_rexxy 0x4f4f
#define MN_rex64z 0x4f55
#define MN_vfpclassbf16z 0x4f5c
#define MN_fldz 0x4f6a
#define MN_vfpclasspdz 0x4f6f
#define MN_vcvtpd2phz 0x4f7b
#define MN_vcvtqq2phz 0x4f86
#define MN_vcvtuqq2phz 0x4f91
#define MN_vfpclassphz 0x4f9d
#define MN_jz 0x4fa9
#define MN_jnz 0x4fac
#define MN_repnz 0x4fb0
#define MN_ccmpnz 0x4fb6
#define MN_loopnz 0x4fbd
#define MN_setnz 0x4fc4
#define MN_ctestnz 0x4fca
#define MN_setzunz 0x4fd2
#define MN_cmovnz (MN_cfcmovnz + 2)
#define MN_cfcmovnz 0x4fda
#define MN_repz 0x4fe3
#define MN_ccmpz 0x4fe8
#define MN_loopz 0x4fee
#define MN_vfpclasspsz 0x4ff4
#define MN_setz 0x5000
#define MN_ctestz 0x5005
#define MN_setzuz 0x500c
#define MN_cmovz (MN_cfcmovz + 2)
#define MN_cfcmovz 0x5013
#define MN_rex64xz 0x501b
#define MN_jecxz 0x5023
#define MN_jcxz 0x5029
#define MN_jrcxz 0x502e
#define MN_rexz 0x5034
#define MN_rexxz 0x5039
#define MN_rex64yz 0x503f
#define MN_rex64xyz 0x5047
#define MN_rexyz 0x5050
#define MN_rexxyz 0x5056
#define MN__disp32_ 0x505d
#define MN__rex2_ 0x5066
#define MN__vex2_ 0x506d
#define MN__vex3_ 0x5074
#define MN__disp16_ 0x507b
#define MN__disp8_ 0x5084
#define MN__load_ 0x508c
#define MN__store_ 0x5093
#define MN__nooptimize_ 0x509b
#define MN__nf_ 0x50a8
#define MN__noimm8s_ 0x50ad
#define MN__rex_ 0x50b7
#define MN__evex_ 0x50bd
#define MN__vex_ 0x50c4
#define MN__insn 0x50ca
