/* This file is automatically generated.  DO NOT EDIT! */
extern ld_emulation_xfer_type ld_elf_x86_64_emulation;
extern ld_emulation_xfer_type ld_elf_i386_emulation;
extern ld_emulation_xfer_type ld_elf32_x86_64_emulation;
extern ld_emulation_xfer_type ld_elf_iamcu_emulation;
extern ld_emulation_xfer_type ld_i386pep_emulation;
extern ld_emulation_xfer_type ld_i386pe_emulation;
extern ld_emulation_xfer_type ld_elf64bpf_emulation;

#define EMULATION_LIST \
  &ld_elf_x86_64_emulation, \
  &ld_elf_i386_emulation, \
  &ld_elf32_x86_64_emulation, \
  &ld_elf_iamcu_emulation, \
  &ld_i386pep_emulation, \
  &ld_i386pe_emulation, \
  &ld_elf64bpf_emulation, \
  0
